#$Id: Celerra.pm,v 1.2 2013/06/18 10:56:15 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::Celerra;
BEGIN {push @INC, "/opt/VRTSperl/lib/site_perl/5.10.0/"};
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Paths;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
require XML::SAX::ExpatXS;
require VRTS::XMLHandler;
use Data::Dumper;

my $ch;
my $ch1;
my $vardir;
my $enchash;
my ($xml, $vdid_obj);
my $probe_timeout;
sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::Celerra;
    $vardir= VRTS::Paths::get_path("AppDir");
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $server = $params->{"server"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    $probe_timeout = $self->{"probe_timeout"};

    my $api_version;
    #### Get Enclosure Capabilities
    my $out = _get_array_capabilities($ch,$server,$user,$pass);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
    if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
    {
        VRTS::Logger::log('error', "Celerra:Unable to contact Celerra Server from primary");
        return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }


##############################################################
# Commenting this part of code. We can revisit exact behavior across all arrays
# in future, if required.
#        if (defined ($ch1) && $ch1 ne "")
#       {
#           print STDERR "Celerra:Unable to contact Celerra from primary, trying from secondary\n";
#           # Contact Celerra server through secondary control host
#           $out = _get_array_capabilities($ch1,$server,$user,$pass);
#           if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
#           {
#                print STDERR "Celerra:Unable to contact Celerra Server from primary as well as secondary\n";
#                return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
#           }
#       }
#       else
#       {
#           print STDERR "Celerra:Unable to contact Celerra Server from primary. secondary not present\n";
#           return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
#       }
       # Make secondary as primary for further run
#       $ch = $ch1;
#   }
#else
#    {
        # Even if primary is working fine, if user has provided secondary
        # Try if it is working fine.
#       if (defined ($ch1) && $ch1 ne "")
#       {
#           $out = _get_array_capabilities($ch1,$server,$user,$pass);
#           if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
#           {
#              print STDERR "Celerra:Unable to contact Celerra Server secondary\n";
#                return undef,&VRTS::Enclosure::Discover::SEC_CONNECT_FAILED;
#           }
#       }
#   }
############################################################
    if( ($out->[1] ne "") && ($out->[1] =~ /403 Forbidden/) )
    {
    	print STDERR "Celerra:Access denied. Error returned[403 Forbidden].\n";
    	return undef,&VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }
    if( ($out->[1] ne "") && ($out->[1] =~ /Receive response failed/)) {
        print STDERR "Celerra:Communication with the provided array IP/Name failed.
            [Receive response failed]\n";
        return undef, &VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    }
    if( ($out->[1] ne "") && ($out->[1] =~ /400 Bad Request/) ) {
        print STDERR "Celerra:Bad Request Error returned[400 Bad Request].\n";
        return undef,&VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    }
    if( ($out->[1] ne "") && ($out->[1] =~ /Could not connect to server/)) {
        print STDERR "Celerra:ERROR: Could not connect to server (err=167)\n";
        return undef, &VRTS::Enclosure::Discover::NO_CONNECTION;
    }
    my $xs = XML::Simple->new();
    $xml  = eval{$xs->XMLin($out->[1])};
    fill_enc_details($xml->{Response});
    return $enchash;

}


sub fill_enc_details
{

    my $response = shift;
    my %val_hash;
    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("Celerra", 0);

    my $vdid = $vdid_obj->get_enclosure_vdid("EMC-Celerra", undef,
		                 $response->{CelerraSystem}->{wwCid});

    $val_hash{VDID} = $vdid;
    $val_hash{serial} = $response->{CelerraSystem}->{serial};
    $val_hash{model} = $response->{CelerraSystem}->{productName};
    $val_hash{displayname} = $response->{CelerraSystem}->{productName};
    $val_hash{product} = $response->{CelerraSystem}->{productName};
    $val_hash{vendor} = "EMC";
    $val_hash{'system-id'} = $response->{CelerraSystem}->{wwCid};

    $enchash->{$vdid} = \%val_hash;
    
}

sub _get_array_capabilities
{
    my $ch = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;

    my $payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");
    _create_login_payload($payload, $server, $user, $pass);
    my $out = VRTS::Enclosure::Utils::push($ch, $payload,
        "\$VARDIR/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
        unlink($payload);
        return $out;
    }

    my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    my $url = "https://$server:443/Login";
    my $login_header = "cel_login";

    create_script($url, "xmlapi.inp", $script, $login_header, 0);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);
    unlink($script);
    unlink($payload);

    if ($out->[0] != 0)
    {
        return $out;
    }
    my @logininfo = $out->[1]; 
    my $token;
    foreach my $line (@logininfo)
    {
        if($line =~ /Set-Cookie: (.*)/) {
            $token = $1;
            chomp($token);
        }
    }
    ### Recieved login token end 

    my $session_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");
    _create_session_payload($session_payload, $token);
    $out = VRTS::Enclosure::Utils::push($ch, $session_payload,
        "\$VARDIR/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
        unlink($session_payload);
        return $out;
    }
    $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    $url = "https://$server:443/servlets/CelerraManagementServices";
    create_script($url, "xmlapi.inp", $script, $login_header, 0);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);

    unlink($script);
    unlink($session_payload);

    ###### Session established end

    if ($out->[0] != 0)
    {
        return $out;
    }
    my $cookie;
    my @arr = $out->[1];
    foreach my $line (@arr) 
    {
        if($line =~ /Set-Cookie: JSESSIONID=(.*)?\;/)
        {
            $cookie = $1;
            $cookie =~ /(.*)\;/;
            $cookie = $1;
        }
    }
    my $probe_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");
    _create_probe_payload($probe_payload, $token, $cookie);
    $out = VRTS::Enclosure::Utils::push($ch, $probe_payload,
        "\$VARDIR/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
        unlink($probe_payload);
        return $out;
    }

    my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    $url = "https://$server:443/servlets/CelerraManagementServices";
    create_script($url, "xmlapi.inp", $script, $login_header, 1);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);

    unlink($probe_payload);
    unlink($script);

    if ($out->[0] != 0)
    {
        return $out;
    }

    my $out1 = _logout($token, $cookie, $server, $ch);
    if (defined($out1) && ($out1->[0] != 0))
    {
        return $out1;
    }

    return $out;

}

sub _logout
{
    my $token =  shift;
    my $cookie = shift;
    my $server = shift;
    my $ch = shift;

    my $logout_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");

    my $logout_header = "cel_logout";
    _create_logout_payload($logout_payload, $token, $cookie);
   my $out = VRTS::Enclosure::Utils::push($ch, $logout_payload,
       "\$VARDIR/xmlapi.inp", $probe_timeout);
   if ($out->[0] != 0)
   {
	   unlink($logout_payload);
	   return $out;
   }
   my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
   my $url = "https://$server:443/servlets/CelerraManagementServices";
   create_script($url, "xmlapi.inp", $script, $logout_header, 0);
   $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);

   unlink($logout_payload);
   unlink($script);

   if ($out->[0] != 0)
   {
	return $out;
   }
   return undef;
}

sub create_script
{
    my $url = shift;
    my $payload = shift;
    my $script = shift;
    my $login_header = shift;
    my $ignore_header = shift;

    open(S, ">$script");
    binmode S;
    print S <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ("/opt/VRTSsfmh/lib/modules") if ( \$^O !~ /Win32/i ) }
require VRTS::Paths;
require VRTS::Util;
require File::Spec;
my \$xprtlc = VRTS::Paths::get_path('xprtlc');
my \$vardir = VRTS::Paths::get_path('AppDir');
my \$payload = File::Spec->catfile(\$vardir, "$payload");
my \$login_header = File::Spec->catfile(\$vardir, "$login_header");
my \$cmd = VRTS::Util::make_command(\$xprtlc, '-j', \$payload, '-l', "$url", '-D', \$login_header);
system(\$cmd);
if($ignore_header eq 0) {
open(my \$handle, "\$login_header");
while(<\$handle>) {
    print \$_;
}
close \$handle;
}
unlink(\$payload);
EOF
    close(S);
}


sub _create_probe_payload
{
    my $payload_file = shift;
    my $token =shift;
    my $connector_sess = shift;

    open (C, ">$payload_file");
    print C <<EOF;
Content-Type: text/xml
Cookie:$token
Cookie: $connector_sess\; Path=/; Secure
CelerraConnector-Sess: $connector_sess

<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<RequestPacket xmlns="http://www.emc.com/schemas/celerra/xml_api">
<Request>
<Query>
<CelerraSystemQueryParams/>
</Query>
</Request>
</RequestPacket>
EOF
    close(C);
}

sub _create_login_payload
{
    my $payload_file = shift;
    my $server = shift;
    my $user = shift;
    my $password = shift;
    
    open (C, ">$payload_file");
    print C <<EOF;
POST http://$server/Login HTTP/1.1
Content-Type:application/x-www-form-urlencoded
Host: $server

user=$user&password=$password&Login=Login
EOF
    close(C);
}

sub _create_session_payload
{
    my $payload_file = shift;
    my $token = shift;

    open (C, ">$payload_file");
    print C <<EOF;
Content-Type:text/xml
Cookie: $token

<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<RequestPacket xmlns="http://www.emc.com/schemas/celerra/xml_api"/>
EOF
    close(C);
}

sub _create_logout_payload
{
    my $payload_file = shift;
    my $server = shift;
    my $token = shift;
    my $connector_sess = shift;

    open (C, ">$payload_file");
    print C <<EOF;
POST http://$server/servlets/CelerraManagementServices HTTP/1.1
Content-Type:Content-Type:text/xml
Host: $server

Cookie:$token
$connector_sess
CelerraConnector-Sess:$connector_sess
CelerraConnector-Ctl: DISCONNECT
EOF
    close(C);
}

1;
