#$Id: HiCommand.pm,v 1.8 2014/03/27 10:54:45 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::HiCommand;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Paths;
require VRTS::Enclosure::Utils;

my $ch;
my $ch1;
my $vardir;
my $user_agent = 'Hitachi:HiCommandCLI:0';
my $probe_timeout;

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
   "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 0, "LDEV_PDEV_MAP" => 0,
   "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 1, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 1,
   "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "SHOW_DEVICE_GROUP_REPLICATION" => 0);


sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::HiCommand;
    $vardir= VRTS::Paths::get_path("AppDir");
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    $probe_timeout = $self->{"probe_timeout"};

    my $api_version;
    #### Get Enclosure Capabilities
    my $out1 = _get_array_capabilities($ch,$server,$port,$user,$pass);
    if ($out1 =~ /^\d+$/)
    {
        my $err = $out1 >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
    if (($out1 eq '') || ($out1 =~ /Server is not/) || ($out1 =~ /Could not connect to server/))
    {
        if (defined ($ch1) && $ch1 ne "")
        {
            VRTS::Logger::log('error', 'Unable to contact HiCommand from primary, trying from secondary');
            # Contact HiCommand through secondary control host
            $out1 = _get_array_capabilities($ch1,$server,$port,$user,$pass);
            if ($out1 eq "" || $out1 =~ /Server is not/)
            {
                 VRTS::Logger::log('error', 'Unable to contact HiCommand Server from primary as well as secondary');
                 return undef,&VRTS::Enclosure::Discover::CONNECT_FAILED;
            }
        }
        else
        {
            VRTS::Logger::log('error', 'Unable to contact HiCommand Server from primary. secondary not present');
            return undef,&VRTS::Enclosure::Discover::CONNECT_FAILED;
        }
        # Make secondary as primary for further run
        $ch = $ch1;
    }
    else
    {
        # Even if primary is working fine, if user has provided secondary
        # Try if it is working fine.
        if (defined ($ch1) && $ch1 ne "")
        {
            my $out1 = _get_array_capabilities($ch1,$server,$port,$user,$pass);
            if ($out1 eq "" || $out1 =~ /Server is not/)
            {
                 VRTS::Logger::log('error', 'Unable to contact HiCommand Server secondary');
                 return undef,&VRTS::Enclosure::Discover::SEC_CONNECT_FAILED;
            }
        }
    }
    if( ($out1 ne "") && ($out1 =~ /Unauthorized/) ) 
    {
        VRTS::Logger::log('error', "Access denied. Error returned [$out1]");
        return undef,&VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }

    if ($out1 =~ /The logged-in user does not have the permissions for using Device Manager functions/) {
        VRTS::Logger::log('error', "Insufficient privileges. Error returned [$out1]");
        return undef,&VRTS::Enclosure::Discover::SECURITY_ROLE_NOT_PRIVILEGE;
    }

    my $xs = XML::Simple->new(keyattr => []);
    my $xml  = eval{$xs->XMLin($out1)};
    if ($@) {
      $api_version = "2.0";
    } else {
      $api_version = $xml->{Response}->{ResultList}->{ServerInfo}->{serverVersion};
      $_ = $api_version;
      if(/v(\d+\.\d+\.\d+)/) {
          $api_version = $1;
      } elsif(/v(\d+\.\d+)/) {
        $api_version = $1;
      }
      else {
          # TODO : Add Error and exit
          VRTS::Logger::log('error', 'Not able to find XML Version. Defaulting to 2.0');
          $api_version = "2.0";
      }
    }

next_run:
    my $payload =  "hicmd.probe.payload";
    _create_probe_payload($payload,$api_version);
    my $out = VRTS::Enclosure::Utils::push($ch, $vardir."/".$payload, "\$VARDIR\/hicmd.probe.payload", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', 'Error in push');
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    my $script = "hicmd.pl";
    my $url = "http://$server:$port/service/StorageManager";
    _create_script($url,$payload,$user,$pass,$script);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, "\$VARDIR\/hicmd.pl", $probe_timeout);
    unlink($vardir."/".$script);
    unlink($payload);
    unlink($vardir."\/hicmd.probe.payload");

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', 'Error in push_exec');
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($out->[1] eq "") {
      return undef, &VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED;
    }
  

    my $out_file = "$vardir/hicmd.probe.out";
    open(O, ">$out_file");
    print O "$out->[1]";
    close(O);

    if ($out->[1] =~ /Server is not up/) {
        VRTS::Logger::log('error', 'Unable to receive data from HiCommand...bailing out');
        return undef, &VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED;
    }

    my %data;
    my $xs = XML::Simple->new();
    my $xml  = eval{$xs->XMLin($out_file)};
    if ($@) {
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }

    my @keys = keys(%{$xml->{Response}->{ResultList}->{StorageArray}});
    my $enchash;
    my $single_array=0;
    foreach my $k (@keys)
    {
        $single_array = 1 if ($k eq "capacityInGB");
        $single_array = 1 if ($k eq "name");
        $single_array = 1 if ($k eq "description");
        $single_array = 1 if ($k eq "arrayFamily");
    }
    if ($single_array == 0)
    {
        foreach my $k (@keys) 
        {
            my $d = $xml->{Response}->{ResultList}->{StorageArray}->{$k};
            my %data;
            my $vdid = %{$d}->{'objectID'};
            my $objid = %{$d}->{'objectID'};

            $data{'serial'} = %{$d}->{'serialNumber'};
            if (defined %{$d}->{'displayArrayType'}) {
              $data{'model'} = %{$d}->{'displayArrayType'};
            } else {
              $data{'model'} = "-";
            }
            $data{'vendor'} = "HITACHI";
            $data{'product'} = %{$d}->{'productName'};
            $data{'displayname'} = $k;
            $data{'api_version'} = $api_version;
            $data{'VDID'} = $vdid;
            $enchash->{$objid} = \%data;
        }
    }
    else
    {
        my $d = $xml->{Response}->{ResultList}->{StorageArray};
        my %data;
        my $vdid = %{$d}->{'objectID'};
        my $objid = %{$d}->{'objectID'};
        $data{'serial'} = %{$d}->{'serialNumber'};
        if (defined %{$d}->{'displayArrayType'}) {
              $data{'model'} = %{$d}->{'displayArrayType'};
        } else {
            $data{'model'} = "-";
        }
        $data{'vendor'} = "HITACHI";
        $data{'product'} = %{$d}->{'productName'};
        $data{'displayname'} = %{$d}->{'name'};
        $data{'api_version'} = $api_version;
        $data{'VDID'} = $vdid;
        $enchash->{$objid} = \%data;
    }
    return $enchash;

}

sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $product = $params->{"product"};
    my $api_version = $params->{"api-version"};
    if ($api_version !~ /\./)
    {
        $api_version = $api_version . "\.0";
    }


    my $payload =  "$vardir/refresh_$$.inp";
    _create_refresh_payload($payload,$api_version,$product,$server,$user,$pass);
    my $out = VRTS::Enclosure::Utils::push($ch, $payload, $payload);
    if ($out->[0] != 0)
    {
         VRTS::Logger::log('error', 'Error in push');
         return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    my $script = "hicmd_refresh.pl";
    my $url = "http://$server:$port/service/StorageManager";
    _create_script($url,$payload,$user,$pass,$script);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, $script);
    unlink($vardir."/".$script);
    if ($out->[0] != 0)
    {
         VRTS::Logger::log('error', 'Error in push_exec');
         return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink($payload);
    if ($out->[1] ne "")
    {
        my $xs = XML::Simple->new(keyattr => []);
        my $xml  = eval{$xs->XMLin($out->[1])};
        if ($@) {
          return &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
        }
        my $status = $xml->{Response}->{EchoCommand}->{status};
        my $ret = $xml->{Response}->{EchoCommand}->{result};
        my $error = $xml->{Response}->{Error}->{description};
        if ($ret eq "0")
        {
            return 0;
        }
        else
        {
            return &VRTS::Enclosure::Discover::CONNECT_FAILED;
        }
    } 
    else
    {
        return &VRTS::Enclosure::Discover::CONNECT_FAILED;
    }

}

sub get_enclosure_details
{
}

sub _get_array_capabilities
{
    my $ch = shift;
    my $server = shift;
    my $port = shift;
    my $user = shift;
    my $pass = shift;

    my $payload =  "xmlapi.inp";
    _create_capability_payload($payload);

    my $out = VRTS::Enclosure::Utils::push($ch, $vardir."/".$payload, "\$VARDIR\/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
         unlink($payload);
         return $out->[0];
    }

    my $script = "hicmd1.pl";
    my $url = "http://$server:$port/service/StorageManager";
    _create_script($url,$payload,$user,$pass,$script);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, "\$VARDIR/hicmd1.pl", $probe_timeout);
    unlink($vardir."/".$script);
    unlink($payload);
    unlink($vardir."/xmlapi.inp");
    unlink($vardir."\/xmlapi.inp");

    if ($out->[0] != 0)
    {
        return $out->[0];
    }

    return $out->[1];
}

sub _create_script
{
    my $url = shift;
    my $payload = shift;
    my $user = shift;
    my $pass = shift;
    my $script = shift;
    
    $payload =~ s/\\/\\\\/g;    

    my $script = $vardir."/".$script;
    open(S, ">$script");
    binmode S;
    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n";
    print S "require VRTS\:\:Paths\;\n";
    print S "my \$xprtlc = VRTS\:\:Paths\:\:get_path(\"xprtlc\")\;\n";
    print S "my \$vardir = VRTS\:\:Paths\:\:get_path(\"AppDir\")\;\n";
    print S "my \$payload = \"\$vardir\/$payload\";\n";
    print S "if ( \$\^O =~ /Win32/i )\{\n";
    print S "my \$bindir = VRTS\:\:Paths\:\:get_path(\"MHBin\")\;\n";
    print S "\$xprtlc =  \"\$bindir\"\.\"\\\\\"\.\"xprtlc.exe\";\n";
    print S "my \$payload1 = \"\$vardir\/$payload\.1\";\n";
    print S "open\(H\,\"\<\$payload\"\)\;\n";
    print S "open\(P\, \"\>\$payload1\"\)\;\n";
    print S "binmode P\; my \$str\;\n";
    print S "while\(\<H\>\)\{\$str \.\= \$_\;\}\n";
    print S "\$str \=\~ s\/\\n\/\\r\\n\/g\;\n";
    print S "print P \"\$str\"\;\n";
    print S "close\(P\)\;close\(H\)\;\n";
    print S "system(\"\\\"\$xprtlc\\\" -j \\\"\$payload1\\\"  -l \\\"$url\\\" -u $user -p $pass\")\;\n";
    print S "unlink\(\$payload1\)\;\n";
    print S "\}else \{\n";
    print S "system(\"\\\"\$xprtlc\\\" -j \\\"\$payload\\\"  -l \\\"$url\\\" -u $user -p $pass\")\;\n";
    print S "\}\n";
    print S "unlink\(\$payload\)\;\n";
    close(S);
}

sub _create_probe_payload
{
    my $payload_file = shift;

    $payload_file = $vardir."/".$payload_file;
    my $api_version = shift;
    open (C, ">$payload_file");
    print C "Content-Type: text/xml\n";
    print C "Accept: */*\n";
    print C "User-Agent: $user_agent\n";
    print C "Connection: close\n\n";
    print C "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    print C "<HiCommandServerMessage>\n";
    print C " <APIInfo version=\"".$api_version."\" />\n";
    print C "  <Request>\n";
    print C "    <StorageManager>\n";
    print C "      <Get target=\"StorageArray\" option=\"all\">\n";
    print C "         <StorageArray>\n";
    print C "        </StorageArray>\n";
    print C "      </Get>\n";
    print C "    </StorageManager>\n";
    print C "  </Request>\n";
    print C " </HiCommandServerMessage>\n";
    close(C);
}

sub _create_capability_payload
{
    my $payload_file = shift;

    my $payload_file = $vardir."/".$payload_file;

    open (C, ">$payload_file");
    print C "Content-Type: text/xml\n";
    print C "Accept: */*\n";
    print C "User-Agent: $user_agent\n";
    print C "Connection: close\n\n";
    print C "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    print C "<HiCommandServerMessage>\n";
    print C " <APIInfo version=\"2.0\" />\n";
    print C "<Request>\n";
    print C "<ServerAdmin>\n";
    print C "<Get target=\"ServerInfo\"/>\n";
    print C "</ServerAdmin>\n";
    print C "</Request>\n";
    print C " </HiCommandServerMessage>\n";
    close(C);
}

sub _create_refresh_payload
{
    my $payload_file = shift;
    my $api_version = shift;
    my $product = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;

    open (C, ">$payload_file");
    print C "Content-Type: text/xml\n";
    print C "Accept: */*\n";
    print C "User-Agent: $user_agent\n";
    print C "Connection: close\n\n";
    print C "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    print C "<HiCommandServerMessage>\n";
    print C " <APIInfo version=\"".$api_version."\" />\n";
    print C "  <Request>\n";
    print C "    <StorageManager>\n";
    print C "      <Add target=\"StorageArray\" >\n";
    print C "         <StorageArray>\n";
    print C "         <CommParameters>\n";
    print C "         <Parameter name=\"arrayFamily\" value=\"$product\"/>\n";
    print C "         <Parameter name=\"ipAddress\" value=\"$server\"/>\n";
    print C "         <Parameter name=\"userID\" value=\"$user\"/>\n";
    print C "         <Parameter name=\"userPassword\" value=\"$pass\"/>\n";
    print C "         </CommParameters>\n";
    print C "        </StorageArray>\n";
    print C "      </Add>\n";
    print C "    </StorageManager>\n";
    print C "  </Request>\n";
    print C " </HiCommandServerMessage>\n";
    close(C);
}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}


1;
