#$Id: NetApp.pm,v 1.10 2014/04/30 10:54:35 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::NetApp;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
use Data::Dumper;
require VRTS::Paths;
require VRTS::Schedule;
require File::Spec;
use VRTS::AddOnInfo;

my $vardir = VRTS::Paths::get_path("AppDir");
my ($xml, $vdid_obj);

my %capabilities = ("VOL_CAPABLE" => 1, "SHOW_RG_LDEV" => 0, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
   "SHOW_REPLICATION" => 0, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 0, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 0, "SHOW_NAS_FS_REPLICATION" => 0,
   "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_VSET" => 0, "SHOW_TP" => 0, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 0,
   "SHOW_PDEV_FOR_THIN_LDEV" => 1, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_QTREE_FILE_CAP" => 0,
   "SHOW_DEVICE_GROUP_REPLICATION" => 1, "STATS_CAPABLE" => 1);

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}
# Initialize the Netapp discovery
sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::NetApp;
    return $module;

}

#
# Get the details about the enclosure
sub get_enclosure_listing
{
    my $self = shift;
    my $creds = shift;
    my $server = $creds->{'server'};
    my $port = $creds->{'port'};
    my $user = $creds->{'user'};
    my $pass = $creds->{'pass'};
    my $ch = $creds->{'control-host'};
    my $probe_timeout = $self->{"probe_timeout"};

    my $clipath = VRTS::Paths::get_path("MHBin");


    my $info = new VRTS::AddOnInfo;
    my $ret = $info->initialize();
    my $installed_version = undef;
    if($ret == 0) {
        my %feature_info = %{$info->get_features()};
        for my $feature (keys %feature_info) {
            next if ($feature_info{$feature}{'ID'} !~ /^VRTSsfmsi-/);
            if ($feature_info{$feature}{'STATUS'} eq "installed" or 
                 $feature_info{$feature}{'STATUS'} eq "enabled")
            {
                 $installed_version = $feature_info{$feature}{'ID'};
            }
        }
    }
    VRTS::Logger::log('debug', "NetApp: The addon installed version is [$installed_version]");

    my $addon_dir = VRTS::Paths::get_path("DeployStoreDir");
    $addon_dir = File::Spec->catfile($addon_dir , $installed_version);
    VRTS::Logger::log('debug', "NetApp: Addon dir is [$addon_dir]");


    $xml = new XML::Simple;
    my $xprtlcmd;
    if ($^O eq "solaris" or $^O eq "linux") {
        $xprtlcmd = "$clipath/xprtlc -l https://localhost/agent/domain/enum | grep '^$ch,'";
    } else {
        $xprtlcmd = "\"$clipath/xprtlc\" -l https://localhost/agent/domain/enum | findstr \"^$ch,\"";
    }

    my $xprtlout = `$xprtlcmd`;

    $xprtlout =~ s/\s+$//;
    my @Domain_info = split(/,/, $xprtlout);
    VRTS::Logger::log('debug', "NetApp: The CH platform is $Domain_info[3].");
    if ($Domain_info[3] !~ /Linux/ && $Domain_info[3] !~ /SunOS/ && $Domain_info[3] !~  /Windows/) {
        VRTS::Logger::log('error', "NetApp: Platform not supported for Netapp discovery");
        return undef, &VRTS::Enclosure::Discover::PLATFORM_UNSUPPORTED;
    }
    if($pass eq "") {
         VRTS::Logger::log('error', "NetApp: Password is empty");
         return undef, &VRTS::Enclosure::Discover::INCORRECT_PASSWORD;
       
    } 

    my ($source, $out);
    if ($Domain_info[3] =~ /Linux/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.linux");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli", $probe_timeout);
        
    }
    if ($Domain_info[3] =~ /SunOS/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.sol");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli", $probe_timeout);
    }
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
   if ($Domain_info[3] =~ /Windows/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.win");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli.exe", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }
        $source = File::Spec->catfile($addon_dir, "arraycli", "libeay32.dll");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/libeay32.dll", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }

        $source = File::Spec->catfile($addon_dir, "arraycli", "ssleay32.dll");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/ssleay32.dll", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }

    }

    my $cli_script = "$clipath/setup_arraycli_perms.pl";
    $out = VRTS::Enclosure::Utils::push_exec($ch,$cli_script,"\$VARDIR\/setup_arraycli_perms.pl", $probe_timeout);
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }

    $cli_script = "$clipath/setup_arraycli.pl";
    $out = VRTS::Enclosure::Utils::push_exec($ch,$cli_script,"\$VARDIR\/setup_arraycli.pl", $probe_timeout);
    if($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
        VRTS::Logger::log('error', "NetApp: Error in running $cli_script");
    }

    my $cmd = "arraycli";
    my $script = "$vardir/netapp.pl"; #"/var/opt/VRTSsfmh/netapp.sh";
    
    # Create the probe script which will be executed by xprtlc
    _create_probe_script($cmd, $server, $user, $pass, $port, "system-get-info", $script);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $script,
        "\$VARDIR\/netapp.pl", $probe_timeout); #$script);
    unlink($script);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', "NetApp: get_enclosure_listing: Error in getting enclosure listing");
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($out->[1] =~ /Error in API invoke/ or $out->[1] =~ /connect to host/ or $out->[1] =~ /not specified/) {
        # Wrong array credential or wrong IP address
        VRTS::Logger::log('error', "NetApp: The array credentials/port or IP address is not correct. Check the creds, IP or Port.");
        return undef, &VRTS::Enclosure::Discover::ARRAY_AUTHORIZATION_FAILED;
    }

    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("NetApp", 0);
     
    my $blob = $out->[1];

    my $file_name = dump_to_file($blob, "NETAPP", "enclosure");
    my $data = $xml->XMLin($file_name);
    my $enc_hash;
    
    # Create the Hash for the enclosure data
    $enc_hash = discover_enc_attributes($data, $enc_hash);

    unlink $file_name;
    return $enc_hash;
}

sub _create_probe_script
{
    my $cmd = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;
    my $port = shift;
    my $api = shift;
    my $script = shift;
    my $vfiler_name = shift;

    if (not defined $vfiler_name or $vfiler_name eq "")
    {
        $vfiler_name = "vfiler0";
    }

    # Remove the space and replace \ with \\
    $cmd =~ s/^\s*//;
    $cmd =~ s/\s*$//;
    $cmd =~ s/\\/\\\\/g;

    open(S, ">$script");
    binmode S;
    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n";
    print S "require VRTS\:\:Paths\;\n";
    print S "my \$vardir = VRTS\:\:Paths\:\:get_path(\"AppDir\")\;\n";
    print S "my \$bindir = VRTS\:\:Paths\:\:get_path(\"MHBin\")\;\n";
    print S "if ( \$\^O !~ /Win32/i ) {\n";
    print S "\$cmd = \"\$bindir\/$cmd\";\n"; 
    print S "print \"not found\" if (\! -x \"\$cmd\")\;\n";
    print S "system(\"\$cmd $server $user $pass $port $vfiler_name Netapp $api\")\;\n}\n";
    print S "else {\n";
    print S "\$cmd = \"\$bindir\"\ . \"\\\\\". \"arraycli\"\;\n";
    print S "print \"not found\" if (\! -x \"\$cmd.exe\")\;\n";
    print S "system(\" \\\"\$cmd.exe\\\"  $server $user $pass $port $vfiler_name Netapp $api\")\;\n}\n";

    #print S "#!/bin/sh\n";
    #print S "$cmd $server $user $pass $port $vfiler_name Netapp $api\n";
    close(S);
}

sub discover_enc_attributes
{
    my $data = shift;
    my $op_hash = shift;

    my %enc_hash;
    my %val_hash;

    my $vdid = $vdid_obj->get_enclosure_vdid("NETAPP",undef,
                                 $data->{'system-info'}->{'system-serial-number'});
    my $old_vdid = $vdid_obj->get_enclosure_vdid("NETAPP",undef,
                                 $data->{'system-info'}->{'system-id'});
    $val_hash{VDID} = $vdid;
    $val_hash{OLDVDID} = $old_vdid;

    $val_hash{serial} = $data->{'system-info'}->{'system-serial-number'};
    $val_hash{model} = $data->{'system-info'}->{'system-model'};
    $val_hash{displayname} = $data->{'system-info'}->{'system-name'};
    $val_hash{product} = $data->{'system-info'}->{'system-model'};
    $val_hash{vendor} = "NetApp";
    $val_hash{'system-id'} = $data->{'system-info'}->{'system-id'};

    $op_hash->{$vdid} = \%val_hash;
    return $op_hash;
}


sub dump_to_file
{
    my $blob = shift;
    my $aid =  shift;
    my $type = shift;

    # TODO: Windows Check
    my $f_name = "$vardir/NETAPP.$aid.$type.xml"; #"/var/opt/VRTSsfmh/NETAPP.$aid.$type.xml";

    open(H, "> $f_name");
    print H $blob;
    close(H);

    return $f_name;
}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}


1;
