#$Id: SYMCLI.pm,v 1.2 2013/06/18 10:56:15 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::SYMCLI;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;

use Data::Dumper;

use constant SYMCLI_MAJOR_VERSION => "7";
use constant SYMCLI_MINOR_VERSION => "0";

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 0, "SHOW_RG_PDEV" => 0, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
    "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 1,
    "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 1, "SHOW_TP" => 1, "SHOW_RG" => 0, "SECONDARY_REMOTE_REP_CAPABLE" => 0,
    "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 1,
    "SHOW_DEVICE_GROUP_REPLICATION" => 0, "STATS_CAPABLE" => 1, "SHOW_TIER" => 0);


my ($xml, $vdid_obj);

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::SYMCLI;
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    my $clipath = $params->{"install-path"};
    my $ch = $params->{"control-host"};
    my $symapi_server = undef;
    $symapi_server = $params->{"server"};
    my $probe_timeout = $self->{"probe_timeout"};

    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("Symmetrix", 0);

    $xml = new XML::Simple;

    $clipath = "/opt/emc/SYMCLI/V7.0.0/bin" if (not defined $clipath or $clipath eq "");

    # Perform a version check for symcfg clis to know if we have the minimum set present.

    my $cmd = "$clipath/symcfg ";
    my $args = " -version -out XML";
    my $vardir= VRTS::Paths::get_path("AppDir");
    my $script = "$vardir/symcli.pl";

    _create_probe_script($cmd, $args, $script, $symapi_server);

    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/symcli.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink $script;

    my $blob = $out->[1];

    if ($blob =~ /network service was not found in the service/)
    {
        return undef, &VRTS::Enclosure::Discover::SYMAPI_SERVER_NOT_VALID;
    }
    elsif ($blob =~ /not found/ or $blob =~ /No such file/)
    {
        # SYMCLI path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }

    my $file_name = dump_to_file($blob, "EMC", "enclosure");
    my $data = $xml->XMLin($file_name);

    my $symcli_version = $data->{SYMCLI}->{version};
    $symcli_version =~ s/^V//;
    my @v_array = split/\./, $symcli_version;

    if ($v_array[0] < &SYMCLI_MAJOR_VERSION)
    {
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
    }

    $cmd = "$clipath/symcfg";
    $args = " list -out XML";
    $script = "$vardir/symcli.pl";
    _create_probe_script($cmd, $args, $script, $symapi_server);
    
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/symcli.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;;
    }
     
    $blob = $out->[1];

    if ($blob =~ /No devices were found/)
    {
        return undef, &VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED;
    }
    elsif ($blob =~ /network service was not found/)
    {
        return undef, &VRTS::Enclosure::Discover::SYMAPI_SERVER_NOT_VALID;
    }

    $file_name = dump_to_file($blob, "EMC", "enclosure");
    $data = $xml->XMLin($file_name);

    my $enc_hash;
    $enc_hash = get_enclosure_details($data, $enc_hash);

    unlink $file_name;
    unlink $script;
    return $enc_hash;
}

sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    my $out;
    my $ch = $params->{"control-host"};
    my $ch1 = $params->{"secondary-ch"};
    my $install_path = $params->{"install-path"};
    my $array_id = $params->{"array-id"};
    my $symapi_server = undef;
    $symapi_server = $params->{"server"};

    return 1 if (not defined $array_id or not defined $install_path 
                             or $array_id eq "" or $install_path eq "");

    my $cmd = "$install_path/symcfg";
    my $args = " discover";
    my $vardir= VRTS::Paths::get_path("AppDir");
    my $script = "$vardir/symcli.pl";
    _create_probe_script($cmd, $args, $script, $symapi_server);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/symcli.pl");
    if ($out->[0] != 0)
    {
         print "Error in push_exec\n";
         if (not defined $ch1 or $ch1 eq "")
         {
             unlink $script;
             return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
         }
         else # control host is up and running fine, we will try using it..
         {
             my $cmd = "$install_path/symcfg";
             my $args = " discover";
             my $vardir= VRTS::Paths::get_path("AppDir");
             my $script = "$vardir/symcli.pl";
             _create_probe_script($cmd, $args, $script, $symapi_server);

             $out = VRTS::Enclosure::Utils::push_exec($ch1, $script, "\$VARDIR\/symcli.pl");
             if ($out->[0] != 0)
             {
                 unlink $script;
                 return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
             }
         }
    }
    unlink $script;

    # symcfg discover is done, now we will verify if the configuration is okay...
    $vardir= VRTS::Paths::get_path("AppDir");
    $array_id =~ s/^EMC//;
    $cmd = "$install_path/symcfg";
    $args = " -sid $array_id verify";
    $script = "$vardir/symcli.pl";
    _create_probe_script($cmd, $args, $script, $symapi_server);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/symcli.pl");
    if ($out->[0] != 0)
    {
        unlink $script;
        return &VRTS::Enclosure::Discover::VERIFICATION_FAILED;
    }

    if ($out->[1] !~ /database file are in sync/)
    {
        unlink $script;
        return &VRTS::Enclosure::Discover::VERIFICATION_FAILED;
    }

    unlink $script;
    return 0;
}

sub get_enclosure_details
{
    my $data = shift;
    my $enc_hash = shift;

    while (my($key,$value) = each(%{$data}))
    {
        if (ref($value) eq 'ARRAY')
        {
            foreach my $val (@$value)
            {
                $enc_hash = _discover_enc_attributes($val, $enc_hash);
            }
        }
        elsif (ref($value) eq 'HASH')
        {
           $enc_hash = _discover_enc_attributes($value, $enc_hash);
        }
    }

    return $enc_hash;
}

sub _discover_enc_attributes
{
    my $data = shift;
    my $op_hash = shift;

    my %enc_hash;
    my %val_hash;

    my $vdid = $vdid_obj->get_enclosure_vdid("EMC",undef,
                                 $data->{Symm_Info}->{symid});
    $val_hash{VDID} = $vdid;
    $val_hash{serial} = $data->{Symm_Info}->{symid};
    $val_hash{model} = $data->{Symm_Info}->{model};
    $val_hash{displayname} = $data->{Symm_Info}->{symid};
    $val_hash{product} = "Symmetrix";
    $val_hash{vendor} = "EMC";
    $val_hash{attachment} = $data->{Symm_Info}->{attachment};

    $op_hash->{$vdid} = \%val_hash;
    return $op_hash;
}


sub dump_to_file
{
    my $blob = shift;
    my $sid =  shift;
    my $type = shift;

    my $vardir= VRTS::Paths::get_path("AppDir");
    my $f_name = "$vardir/EMC.$sid.$type.xml";

    open(H, "> $f_name");
    print H $blob;
    close(H);

    return $f_name;
}

sub _create_probe_script
{
    my $cmd = shift;
    my $args = shift;
    my $script = shift;
    my $symapi_server = shift;

    $cmd =~ s/^\s*//;
    $cmd =~ s/\s*$//;
    $cmd =~ s/\\/\\\\/g;

    open(S, ">$script");
    binmode S;
    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n";

    if (defined $symapi_server and $symapi_server ne "")
    {
        print S "\$ENV\{\'SYMCLI_CONNECT_TYPE\'\} = \"REMOTE\"\;\n";
        print S "\$ENV\{\'SYMCLI_CONNECT\'\} = \"$symapi_server\"\;\n";
    }

    print S "if ( \$\^O !~ /Win32/i ) {\n";
    print S "print \"not found\" if (\! -x \"$cmd\")\;\n";
    print S "system(\"$cmd $args\")\;\n}\n";
    print S "else {\n";
    print S "print \"not found\" if (\! -x \"$cmd.exe\")\;\n";
    print S "system(\" \\\"$cmd.exe\\\" $args\")\;\n}\n";
    close(S);
}

sub provide_capabilities
{
    my $self = shift;
    return \%capabilities;
}


1;
