#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::VPLEX;
use strict;
use Carp;
use JSON;
require Exporter;
require XML::Simple;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
use Data::Dumper;
use File::Copy;

use vars qw($VERSION @ISA @EXPORT );
$VERSION = '5.00'; 
@ISA = qw(Exporter);
@EXPORT = qw(create_vplex_script 
             vplex_debug
             vplex_error   
             get_vplex_clusters 
             get_vplex_config_dump
             print_json
             push_exec_vplex_api
             push_vplex_payload
             set_dbg
);


use File::Spec qw/catfile/;

my $log_init = 1;
my $probe_timeout;
our $f_dbg=0;
sub new 
{
    set_dbg();
    my $class = shift;
    my $self = {};
    bless ($self, $class);
    return $self;
}
sub provide_capabilities
{
    return undef;
}
sub initialize
{
    my $top = shift;
    my $module = new VRTS::Enclosure::Discover::VPLEX;
    return $module;
}
sub get_enclosure_listing
{
    my ($self,$params) = @_;
    my $enc_hash= {};
    my ($data, $e, $pld_fl, $r );
    my @clusters = ();
    vplex_debug("get_enclosure_listing");
    my $p_ch = $params->{"control-host"};
    my $host = $p_ch;
    my $s_ch = $params->{"secondary-ch"};
    my $server = $params->{"server"};
    my $port = (defined ($params->{"port"}))? $params->{"port"}:443;
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    $probe_timeout = $self->{"probe_timeout"};
    
    if ($log_init)
    {
        vplex_debug("VRTS::Enclosure::Discover::VPLEX::Initialized log\n");
        $log_init=0;
    }
    vplex_debug("Primary CH: $p_ch"); 
    vplex_debug("Secondary CH: $s_ch");
    vplex_debug("Server: $server");
    vplex_debug("Port: $port");
    vplex_debug("User: $user");

    # get vplex clusters. Need to add sencondary in case of failure
    ($r, @clusters) = get_vplex_clusters($user, $pass, $server, $port, $p_ch);
    if($r)
    {
        ($r, @clusters) = get_vplex_clusters($user, $pass, $server, $port, $s_ch) if (defined $s_ch && $s_ch ne "");
        return ($enc_hash, $r)  if($r);
        $host=$s_ch;
    }
    return ($enc_hash,$r)  if ($r);
    vplex_debug("get_enclosure_listing\n".join("\n",@clusters));

    ($r, $enc_hash) = _get_enclosure_details($user, $pass, $server, $port, $host, @clusters);
    vplex_debug("_get_enclosure_details rc $r");

    return ($enc_hash, $r) if ($r);
    return ($enc_hash, &VRTS::Enclosure::Discover::PROBE_SUCCESS)

}

# enables debug flag if VOM_DISCOVERY_DEBUG is set to 1
sub set_dbg
{
    $f_dbg = 1 if (defined $ENV{'VOM_DISCOVERY_DEBUG'} && $ENV{'VOM_DISCOVERY_DEBUG'} == 1);
    my $file =  File::Spec->catfile(VRTS::Paths::get_path("AppDir"), "tmp", "vxdebug");
    $f_dbg = 1  if (-e $file);
}
sub vplex_debug
{
    if ($f_dbg)
    {
        my $fmt = shift;
        my $t = sprintf ($fmt, @_);
        VRTS::Logger::log("debug", $t);
    }
}
sub vplex_error
{
    my $fmt = shift;
    my $t = sprintf ($fmt, @_);
    VRTS::Logger::log("error", $t);
}

#
# Creates the payload file for VPLEX HTTP API call and pushes it to the control host
#
my $pld_cnt=0;
sub push_vplex_payload
{    my ($user, $pwd, $ch, $args)=@_;
     vplex_debug("push_vplex_payload arg: $ch $args ");

     my $p_fl = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),"VPLEX.$$.pld.lcl");
     my $r = open(S, ">$p_fl");
     if (!$r)
     {
         vplex_error ("Failed to open file $p_fl.\n    $!");
         return (&VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR, undef, "Failed to open file $p_fl.");
     }
     binmode S;
     my $text = <<END;
Content-Type:text/json
Username:$user
Password:$pwd

END


print S $text;
     if (defined $args)
     {
         $text = '{"args":"'.$args.'"}';
         print S $text;
     }
     
     close(S);
     my $out = VRTS::Enclosure::Utils::push($ch, $p_fl, "\$VARDIR\/VPLEX.$$.$pld_cnt.pld", $probe_timeout);
     unlink($p_fl);
     if ($out->[0] != 0)
     {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            vplex_error ( "Request timed out due to no activity for timeout period\n   $out->[1]");
            return(&VRTS::Enclosure::Discover::SESSION_TIMEDOUT,"VPLEX.$$.pld", $out->[1]);
        }
        vplex_error ("Failed to push file $p_fl to the remote host.\n    $out->[1]");
        return(&VRTS::Enclosure::Discover::MH_NOT_REACHABLE,"VPLEX.$$.pld", $out->[1]);
     }
     my $f = "VPLEX.$$.$pld_cnt.pld";
     $pld_cnt++;
     return (0,$f,undef);
}

#
#  Creates vplex API script to be executed. Inputs URL & payload info that contains 
#  user name, password and API arguments
#
sub create_vplex_script
{
    my ($url,$pld_fl, $fGet) =@_;
    vplex_debug("create_vplex_script url $url");
    my $file = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),"VPLEX.$$.pl");

    my $r = open(S, ">$file");
    if (!$r)
    {
        vplex_error ("Failed to open file $file.\n    $!");
        return (1,undef);
    }

    binmode S;
    print S '#!/opt/VRTSsfmh/bin/perl'."\n";
    print S 'BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }'."\n";
    print S 'require VRTS::CMD;'."\n";
    print S 'require VRTS::Paths;'."\n";
    print S 'my $xprtlc = &VRTS::CMD::XPRTLC;'."\n";
    print S 'my $vardir = VRTS::Paths::get_path("AppDir");'."\n";
    print S 'my $pld_fl = $vardir."'.$pld_fl.'";'."\n";
    if ($fGet)
    {
        print S 'my $cmd = "$xprtlc -j \"$pld_fl\" -l '.'\"'.$url.'\"'.' -m GET";'."\n";
    }
    else
    {
        print S 'my $cmd = "$xprtlc -j \"$pld_fl\" -l '.'\"'.$url.'\"'.'";'."\n";
    }
    print S 'my $r = open(OUT, "$cmd |");'."\n";
    print S 'while (<OUT>) { printf ($_);};'."\n";
    print S 'close(OUT);'."\n";
    print S 'unlink($pld_fl);'."\n";
    print S 'exit($r);'."\n";
    close(S);
    return (0,$file);
}
#
#  Creates vplex API script to be executed. Inputs URL & payload info that contains 
#  user name, password and API arguments
#

sub create_halssh_script
{
    my ($user, $server, $passwd,$halcmd ) =@_;
    vplex_debug("create_halssh_script cmd  $halcmd");
    my $file = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),"VPLEX.halssh.$$.pl");

    my $r = open(S, ">$file");
    if (!$r)
    {
        vplex_error ("Failed to open file $file.\n    $!");
        return (1,undef);
    }
    #./halssh -l admin -pw 'Password' -4 10.182.67.46 'cat /var/log/VPlex/cli/cluster-2.xml'
    binmode S;
    print S '#!/opt/VRTSsfmh/bin/perl'."\n";
    print S 'BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }'."\n";
    print S 'require VRTS::Util;'."\n";
    print S '$ENV{'."'".'LD_LIBRARY_PATH'."'".'}='."'".'/opt/VRTSsfmh/lib\''." if ( ".'$^O'." !~ /Win32/i );"."\n";
    print S 'my $passwd ='."'".$passwd."';"."\n";
    print S 'my $cmd = File::Spec->catfile(VRTS::Paths::get_path("MHBin"),"halssh");'."\n";
    print S 'if ( $^O =~ /Win32/i )'."\n";
    print S '{'."\n";
    print S '    $cmd="\"".$cmd.".exe\"";'."\n";
    print S '    $cmd = $cmd." -l '.$user.' -pw \"".$passwd."\" -4 '.$server.' \"'.$halcmd.'\"";'."\n";
    print S '}'."\n";
    print S 'else'."\n";
    print S '{'."\n";
    print S '    $cmd = $cmd." -l '.$user.' -pw \'".$passwd."\' -4 '.$server.' \''.$halcmd."'\";"."\n";
    print S '}'."\n";
    print S 'my $r = open(OUT, "$cmd |");'."\n";
    print S 'if (!$r) {printf ("Failed to execute the command $cmd\n"); exit(1);} '."\n";
    print S 'while (<OUT>) { printf ($_);};'."\n";
    print S 'close(OUT);'."\n";
    print S 'exit(0);'."\n";
    close(S);
    return (0,$file);
}
sub _cnvrt_to_json
{
    my $out= shift;
    
    $out->[1] =~ s/\r//g;
    my @t = split(/\n/, $out->[1]);
    if ($f_dbg)
    {
        # some outputs are one long string. spliting to 80 characters a line
        vplex_debug("Out[0]\n".$out->[0]);
        my @t1 = $out->[1] =~ /(.{80})/g;
        vplex_debug("Out[1]\n".join("\n",@t1));
        @t1=();
    }
    #
    # output has some junk characters befor the JSON result. stripe it and copy it. Can do some optimization by poping of the array elements
    #
    while($t[0] ne "{")
    {
        shift(@t);
        if (!defined $t[0])
        {
            vplex_error("SysError:Unsupported data stream Missing opening brace");
            return undef;
        }
    }
    my @t1=();
    foreach (@t)
    {
        push @t1, $_;
        last if ($_ eq "}");
    }
    my $stg = join("\n", @t1);
    my $data=undef;
    eval {
        $data = from_json($stg);
    };
    if ( $@)
    {
        vplex_error("Failed to convert to json.Bad JSON\n$@");
        vplex_error("$@");
        return undef;
    }
    return $data; 
}
sub _wait_for_completion
{
    my $cnt=0;
    my ($ch,$server,$url,$user,$pass)=@_;
    my ($msg, $json, $out, $res, $msg,$script);
    my ($r, $pld_fl, $e);
            
    do
    {
        vplex_debug("_wait_for_completion push_vplex_payload");
        ($r, $pld_fl, $e) = push_vplex_payload($user,$pass,$ch, "clusters");
        if ($r)
        {
            vplex_error("_wait_for_completion: Failed to push payload file. $e ");
            return(1,"");
        }       
        vplex_debug("_wait_for_completion creating script");
        my ($r, $script) = create_vplex_script($url, $pld_fl, 1);
        sleep(15);
        $cnt++;
        $out = VRTS::Enclosure::Utils::push_exec($ch, $script, '$VARDIR/VPLEX'."$$.$server.pl", 3600);
        #unlink($script);
        if ($out->[0] != 0)
        {
            vplex_error("Error in push_exec\n$out->[1]");
            my $err = $out->[0] >> 8;
            if ($err == 192)            
            {
                vplex_error ( "Request timed out due to no activity for timeout period\n ");
            }
            else
            {
                vplex_error("Error in push_exec\n$out->[1]");
            }
            unlink($pld_fl); 
            return(1,"");
        }
        vplex_debug("_wait_for_completion out \n $out->[1]");
        $json = _cnvrt_to_json($out);
        if (defined $json)
        {
            $res = $json->{response}; 
            $msg = $res->{message};
            vplex_debug("_wait_for_completion except $res->{exception}");
            if (defined $res->{exception})
            {
                vplex_error("_wait_for_completion Exception $res->{exception} ");     
                unlink($pld_fl);
                return (1, "");
            }
            else
            {
                vplex_debug("_wait_for_completion msg $msg");
                if ($msg =~ /Command execution continues to be under progress/)
                {
                     vplex_debug("_wait_for_completion count:$cnt");
                }
                elsif ($msg !~  m/execution taking longer than 60 seconds/ )
                {
                    unlink($pld_fl);
                    return (0, "")
                }
            }    
        }
        else
        {
            vplex_error("_wait_for_completion json not defined");
        } 
        vplex_debug("_wait_for_completion count:$cnt");
        unlink($pld_fl);
    }while($cnt<45); 
    unlink($pld_fl);
    return (1, "");
}
#
# executes script file that has invocations to VPLEX API on 
# the control host and returns the output
#
sub push_exec_vplex_api
{
    my ($ch, $script,$server, $pld_fl, $user,$pass)=@_;
    vplex_debug("_push_exec_vplex_api CH $ch");
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, '$VARDIR/VPLEX'."$$.$server.pl", 3600);
    if ($out->[1] =~ /Could not connect to server/ ||
        $out->[1] =~ /404 Not Found/)
    {
        vplex_error("Error in push_exec\n$out->[1]");
        return (&VRTS::Enclosure::Discover::ARRAY_AUTHORIZATION_FAILED, undef);
    }
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            vplex_error ( "Request timed out due to no activity for timeout period\n ");
            return(&VRTS::Enclosure::Discover::SESSION_TIMEDOUT,undef);
        }
         vplex_error("Error in push_exec\n$out->[1]");
         return (&VRTS::Enclosure::Discover::MH_NOT_REACHABLE,undef);
    }
    if ($out->[1] eq "" ||
        $out->[1] =~ /Server is not up/) 
    {
      return (&VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED, undef);
    }

    $out->[1] =~ s/\r//g;

    my @t = split(/\n/, $out->[1]);
    if ($f_dbg)
    {
        # some outputs are one long string. spliting to 80 characters a line
        vplex_debug("Out[0]\n".$out->[0]);
        my @t1 = $out->[1] =~ /(.{80})/g;
        vplex_debug("Out[1]\n".join("\n",@t1));
        @t1=();
    }
    #
    # output has some junk characters befor the JSON result. stripe it and copy it. Can do some optimization by poping of the array elements
    #
    my $loc=undef;
    while($t[0] ne "{")
    {
        if ($t[0] =~ /^Location\:/ && !defined $loc)
        {
            $loc =$t[0];
            $loc =~ s/^Location\: //g;
            vplex_debug("command location $loc");
            return _wait_for_completion($ch, $server, $loc,$user,$pass);
        }
        shift(@t);
        if (!defined $t[0])
        {
            vplex_error("SysError:Unsupported data stream Missing opening brace");
            return (&VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED,undef);
        }
    }
    
    my $i=0;
    my @t1=();
    foreach (@t)
    {
        push @t1, $_;
        last if ($_ eq "}");
    }
    $out->[1] = join("\n", @t1);

    my $data;
    # convert to perl object
    eval {
        $data = from_json($out->[1]);
    };
    if ( $@)
    {
        vplex_error("Failed to convert to json.Bad JSON\n$@");
        vplex_error("$@");
        $out=undef;
        return (&VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR,undef);
    }
    
    my $res = $data->{response}; 
    my $msg = $res->{message};
    if (defined $res->{exception})
    {
        if ($res->{exception} ne "null" &&  $res->{exception} ne "")
        {
            vplex_error("VPLEX exception:\n $out->[1]\n");
            undef $data;
            $out=undef;
            return (&VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED, undef);            
        }
    }
    $out=undef;
    if ($msg =~  m/User authentication failed/)
    {
        undef $data;
        return (&VRTS::Enclosure::Discover::AUTHENTICATION_FAILED, undef);            
    }
    vplex_debug("VPLEX: custom data\n".$res->{'custom-data'});
    return (0, $res->{'custom-data'});
}
#
# Returns a list VPLEX cluster name. USE vxplexcli command
#
# VPlexcli:/> ls  clusters
#
#/clusters:
#cluster-1  cluster-2
#
sub get_vplex_clusters
{
    my ($user, $pass, $server, $port, $p_ch)=@_;
    vplex_debug("Enter:get_vplex_clusters $p_ch "); 

    my @clusters = ();
    my $script;
    my ($r, $pld_fl, $e) = push_vplex_payload($user,$pass,$p_ch, "clusters");
    return ($r, @clusters) if ($r);
    my $url = 'https://'.$server.':'.$port.'/vplex/ls';
    ($r, $script) = create_vplex_script($url, $pld_fl);
    return ($r, @clusters) if ($r);    
    my $data;
    ($r,$data) = push_exec_vplex_api($p_ch,$script, $server, $pld_fl,$user,$pass);

    unlink($script);

    return ($r, @clusters) if ($r);
    @clusters = split("\n", $data);
    undef $data;
    my $stg = $clusters[2];
    @clusters = split(/\s+/, $stg);
    @clusters = sort(@clusters);
    vplex_debug("Discovered clusters from host $p_ch:\n".join("\n", @clusters));
    return (0, @clusters);
}
#
#  retrives VPLEX configuration dump file for the specified cluster. 
#  Data is returned in XML format. Converts it and return it in json format 
#  Executes VPLEX API call and then calls halssh to retrieves the file
#
my $dump_data={};
sub get_vplex_config_dump
{
    my ($user, $passwd, $server, $port, $p_ch, $cluster)=@_;
    my ($e, $e_pwd, $pld_fl, $r );
    #
    # Call vplexcli --cluster $cluster --file $cluster.xml
    #
    return (0, $dump_data->{$cluster}) if (defined $dump_data->{$cluster});
    my $f = '/tmp/'."$p_ch".".$cluster".".xml";
    my ($r, $pld_fl, $e) = push_vplex_payload($user,$passwd,$p_ch, "--cluster $cluster  --file $f"); 
    return ($r, undef) if ($r);
    my $url = 'https://'.$server.':'.$port.'/vplex/cluster+configdump';

    my $script;
    ($r, $script) = create_vplex_script($url, $pld_fl);
    return ($r, undef) if ($r);
    my @lines=();
    ($r, @lines) = push_exec_vplex_api($p_ch,$script, $server, $pld_fl,$user,$passwd);
    undef @lines;
    return ($r, undef) if ($r);
    unlink($pld_fl);
    unlink($script); 
    # get the xml file. Use halssh
    ($r, $script) = create_halssh_script($user, $server, $passwd, 'cat '.$f.'; rm '.$f);
    #($r, $script) = create_halssh_script($user, $server, $passwd, 'cat '.$f);
    return ($r, undef) if ($r);
    my $out = VRTS::Enclosure::Utils::push_exec($p_ch, $script, '$VARDIR/VPLEX.halssh.'."$$.$server.pl", 3600, 2);
    unlink($script); 
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            vplex_error ( "Request timed out due to no activity for timeout period\n ");
            return(&VRTS::Enclosure::Discover::SESSION_TIMEDOUT,undef);
        }
         vplex_error("Error in push_exec\n$out->[1]");
         return (&VRTS::Enclosure::Discover::MH_NOT_REACHABLE,undef);
    }
cont:
    vplex_error("Processing xml:\n $out->[1]");
    
    my $data;
    eval 
    {
        my $xml = new XML::Simple;
        $data = $xml->XMLin($out->[1]);
    };
    if ($@)
    {
        vplex_error("Failed to convert xml data. Possible network error or out of FS space. Cluster $cluster. File $f\n $@"); 
        my $to = "VPLEX_".$server."_$cluster"."_config_dump.xml";
        $to = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),$to);
        move($out->[1], $to);
        return (&VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE,undef);	
    }
    if (!$f_dbg)
    {
        unlink($out->[1]);
        goto out;
    }
#Dump data for debugging
    $dump_data->{$cluster} = $data;
    vplex_debug("Dump data:\n");
    my $fl = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),$cluster.".json");
    open(FH,'>'."$fl");
    eval 
    {
        my $js = to_json($data, {pretty => 1, indent => 4});
        print FH $js;
    };
    if ($@)
    {
        vplex_error("Failed to convert json to string .$@"); 
        return (&VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE,undef);
    }
    my $to = "VPLEX_".$server."_$cluster"."_config_dump.xml";
    $to = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),$to);

    move($out->[1], $to); 
    close FH;
out:
    undef $out;
    #vplex_error("Retuning $r");
    return (0, $data);
}
#
#  returns a hash of enclosure details
#
sub _get_enclosure_details
{
    my ($user, $pass, $server, $port, $p_ch, @clusters) = @_;
    my $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("Vplex", 0);
    my $arrayfamily = "VPlex";
    my $enc_hash = {};
    my @s_clusters = sort(@clusters);
    my @e_vdids=();
    my $clus;
    foreach (@s_clusters)
    {
        my ($r, $data) = get_vplex_config_dump($user, $pass, $server, 
                                               $port, $p_ch, $_);
        $clus = $data->{cluster}[1] if (!$r);
        if ($r || !defined $clus->{'top-level-assembly'})
        {
            if ($r == 0)
            {
                vplex_error("Missing enclosure VDID");
            }
            else
            {
                vplex_error("Failed to get the configuration information for the node $_");
            }
            return (&VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED,$enc_hash);
        }
        push @e_vdids , $clus->{'top-level-assembly'}
    }
    my $enc_detail ={};

    my $e_vdid = $vdid_obj->get_enclosure_vdid("EMC",$arrayfamily,join("_",@e_vdids));
    $enc_detail->{VDID} = $e_vdid;
    $enc_detail->{serial} = $e_vdid;
    $enc_detail->{displayname} = 'VPLEX@'.$server;
    $enc_detail->{version} = $clus->{'version'};
    $enc_detail->{id} = $enc_detail->{VDID};
    $enc_detail->{product} = "VPLEX";
    $enc_detail->{vendor} = "EMC";
    $enc_detail->{model} = "VPLEX";
    $enc_hash->{$e_vdid} = $enc_detail;
    my $stg;
    vplex_debug("Enclosure details:\n");
    print_json($enc_hash);
    return (0, $enc_hash);
}
sub print_json
{
    eval 
    {
        my $js = to_json(shift, {pretty => 1, indent => 4});
        vplex_debug($js);
    };
    if ($@)
    {
        vplex_error("Failed to convert json to string .$@"); 
    }
    return (0, "");
}

1;
