#$Id: XIV.pm,v 1.4 2014/04/29 09:26:03 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::XIV;
use strict;
use Carp;
require Exporter;
require VRTS::Paths;
require VRTS::Enclosure::Utils;
require VRTS::Enclosure::VDID;

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 0, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
                     "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 0, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 0, "LDEV_PDEV_MAP" => 0,
                     "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 0, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 0,
                     "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0,
                     "SHOW_DEVICE_GROUP_REPLICATION" => 0, "STATS_CAPABLE" => 1);

my $vardir;
my $script;
my $dscript;
$vardir= VRTS::Paths::get_path("AppDir");
$script = "$vardir/xivscript$$.pl";
$dscript = "$vardir/dxivscript$$.pl";

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;
    my $module = new VRTS::Enclosure::Discover::XIV;
    return $module;

}

sub test_exec_xcli
{
    my $ch = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;
    my $path = shift;
    my $cmd = shift;
    my $rc = 0;

    _create_script_for_command($server,$user,$pass,$path,'config_get',$script,'csv');
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript.pl");
    if ($out->[0] != 0 or $out->[1] =~ /not found/i)
    {
         unlink($script);
         return 1;
    }
    unlink($script);
    return 0;
}

sub exec_xcli
{
    my $ch = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;
    my $path = shift;
    my $cmd = shift;
    my $format = shift;
    my $rc = 0;

    _create_script_for_command($server,$user,$pass,$path,$cmd,$script,$format);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript$$.pl");
    if ($out->[0] != 0  or $out->[1] =~ /not found/i)
    {
         exit 0;
    }
    unlink($script);
    return $out;
}
sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    my $ch = $params->{"control-host"};
    my $server = $params->{"server"};
    my $server2 = $params->{ARRAY_XIV_IP2};
    my $server3 = $params->{ARRAY_XIV_IP3};
    if (defined($server2) && ($server2 ne '')) {
        VRTS::Logger::log('debug', "server2 = $server2");
        $server .= ',' . $server2; 
    }
    if (defined($server3) && ($server3 ne '')) {
        VRTS::Logger::log('debug', "server3 = $server3");
        $server .= ',' . $server3; 
    }
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $path = $params->{"install-path"};
    my $probe_timeout = $self->{"probe_timeout"};
    my $rc = 0;

    _create_script_for_command($server,$user,$pass,$path,"config_get",$script);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    elsif ($out->[1] =~ /not found/i || $out->[1] =~ /No such file/i)
    {
         return undef,&VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }
    elsif ($out->[1] =~ /Incorrect password/i)
    {
         return undef,&VRTS::Enclosure::Discover::INCORRECT_PASSWORD;
    }
    elsif ($out->[1] =~ /USER_NAME_DOES_NOT_EXIST/i)
    {
         return undef,&VRTS::Enclosure::Discover::USER_NAME_DOES_NOT_EXIST;
    }
    elsif ($out->[1] =~ /NO_CONNECTION/i)
    {
         return undef,&VRTS::Enclosure::Discover::NO_CONNECTION;
    }
    elsif ($out->[1] =~ /LOGIN_FAILURE_USER_FAILED_TO_LOGIN/i)
    {
        return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }

    unlink($script);

    my %data1;
    my $objid;
    my $enchash;
    if ($out->[0] == 0)
    {
        my @data = split ("\n",$out->[1]);
        foreach my $d (@data)
        {
            $d =~ s/\"//g;
            my @arr = split /,/,$d;
            $data1{'displayname'} = $arr[1]  if ($arr[0] eq "system_name");
            $data1{'serial'} = $arr[1]  if ($arr[0] eq "system_id");
            $data1{'model'} = $arr[1]  if ($arr[0] eq "machine_model");
            $data1{'type'} = $arr[1]  if ($arr[0] eq "machine_type");
            $data1{'vendor'} = "IBM";
            $data1{'product'} = "IBM XIV";
            $data1{'VDID'} = $arr[1]  if ($arr[0] eq "system_name");
            $objid = $arr[1]  if ($arr[0] eq "system_name");
        }

        my $vdid_obj = new VRTS::Enclosure::VDID ();
        $vdid_obj->initialize("XIV", 0);
        my ($vdid, $oldvdid)  = 
                $vdid_obj->get_enclosure_vdid("IBM",$data1{'type'},$data1{'serial'});

        if (defined $oldvdid and $oldvdid ne $vdid)
        {
            $data1{'OLDVDID'} = $oldvdid;
        }
        $data1{'VDID'} = $vdid;
        $enchash->{$vdid} = \%data1;
    }

    return $enchash;
}

sub _create_script_for_command
{
    my $server = shift;
    my $user = shift;
    my $pass = shift;
    my $path = shift;
    my $cmd = shift;
    my $script = shift;
    my $format = shift;
    my $iplist = '';
    my $cli;
    my $args;

    $path =~ s/\\/\\\\/g;
    $cmd=~ s/\\/\\\\/g;
    $cmd =~ s/^\s*//;
    $cmd =~ s/\s*$//;
    $cli = $path . "/xcli";

    if($server =~ /,/)
    {
        my @ips = split(/,/, $server);
        foreach(@ips)
        {
            $iplist .= " -m $_";
        }
    }
    else
    {
        $iplist = "-m $server";
    }

    if($format eq 'xml')
    {
        $args = " -x -y -u $user -p $pass $iplist $cmd";
    }
    else
    {
        $args = " -s -y -u $user -p $pass $iplist $cmd";
    }

    open(S, ">$script");
    binmode S;
    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n";
    print S "if ( \$\^O !~ /Win32/i ) {\n";
    print S "print \"not found\" if (\! -x \"$cli\")\;\n";
    print S "system(\"$cli $args\")\;\n}\n";
    print S "else {\n";
    print S "print \"not found\" if (\! -x \"$cli.exe\")\;\n";
    print S "system(\" \\\"$cli.exe\\\" $args\")\;\n}\n";
    close(S);

}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}

1;
