#$Id: VDID.pm,v 1.1 2012/09/19 05:54:50 vkaul Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::VDID;
use strict;
use Carp;
require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $TO_DEFAULT);

@ISA = qw(Exporter);
@EXPORT = qw(new initialize switch get_enclosure_vdid get_disk_vdid);
$VERSION = '3.0';

$TO_DEFAULT = 0;

sub new 
{
    my $class = shift;
    my $self = {};

    # configuration options
    $self->{CONFIGOPTIONS} = {};
    $self->{PLUGIN} = {};
    $self->{HAVE_PLUGIN} = 0;

    bless ($self, $class);
    return $self;
}

sub initialize 
{
    my $self = shift;
    my $vendor = shift;
    my $to_default = shift;

    $TO_DEFAULT = 1 if (defined $to_default and $to_default == 1);
    
    # Based on the vendor information that is passed, 
    #  setup the lower level module.
    my $module_name = "VRTS::Enclosure::VDID::".$vendor;
    if (eval "require $module_name")
    {
        my $vendor_handle  = &{\&{"${module_name}::initialize"}};
        $self->{HAVE_PLUGIN} = 1;
        $self->{PLUGIN} = $vendor_handle;
    }
    else
    {
        croak "$@ module [$vendor] for [$module_name] not available\n";
    }
}

sub switch
{
    my $self = shift;
    my $vendor_class = shift;
}

sub get_enclosure_vdid
{
    my $self = shift;
    my $vendor = shift;
    my $product = shift;
    my $cabsno = shift;

    return undef if (not defined $self);
    return undef if ($self->{HAVE_PLUGIN} == 0 and $TO_DEFAULT == 0);

    if (defined $self->{PLUGIN})
    {
        return $self->{PLUGIN}->get_enclosure_vdid($vendor, $product, $cabsno);
    }
    elsif ($TO_DEFAULT == 1)
    {
        # Route the call to the default implementation....
    }

    return undef;
}

sub get_disk_vdid
{
    my $self = shift;
    my $vendor = shift;
    my $product = shift;
    my $cabsno = shift;
    my $serial = shift;
    my $is_pdev = shift;
    my $pdev_sno = shift;

    return undef if (not defined $self);
    return undef if ($self->{HAVE_PLUGIN} == 0 and $TO_DEFAULT == 0);

    if (defined $self->{PLUGIN})
    {
        return $self->{PLUGIN}->get_disk_vdid($vendor, $product, $cabsno, $serial, $is_pdev, $pdev_sno);
    }
    elsif ($TO_DEFAULT == 1)
    {
        # Route the call to the default implementation....
    }

    return undef;
}

1;
