#$Id: Hitachi.pm,v 1.3 2013/06/21 07:11:42 aditya Exp $
#$copyright$

package VRTS::Enclosure::VDID::Hitachi;
use strict;
use Carp;
require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $TO_DEFAULT);

sub new
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::VDID::Hitachi;
    return $module;

}


sub get_disk_vdid
{
    my $self = shift;
    my $vendor = shift;
    my $product = shift;
    my $cabsno = shift;
    my $serial = shift;
    my $is_pdev = shift;
    my $devnum = shift;

    my $vdid = undef;
    $product =~ s/HDS//g;
    $vdid = generate_ldev_vdid ($vendor,$product,$serial,$devnum);
    return $vdid;
}

#############
# Logic:
#
# 1. Get Disk CU Number
# 2. Get LDEV Number
# 3. Construct VDID
###################

sub
generate_ldev_vdid
{
    my $vendor = shift;
    my $product = shift;
    my $serial = shift;
    my $devnum = shift;

    my $cu = _get_cu($devnum);
    my $ldev = _get_ldev_id($devnum);
    if ($product =~ m/^95/)
    {
        #It is a 9500V model, so strip the serialnumber of 6xxx and use just last 4 bytes from the 8 byte serialnumber.
        $serial= substr $serial,4,4;
    }
    # Now Strip off any leading zeroes.
    $serial=~ s/^0+//;
    my $vdid = $vendor."_".$product."_".$serial. "_" . $cu . "_" . $ldev;
    return $vdid;
}


# Compute and return CU id in hex.
sub
_get_cu()
{
    my $devnum = shift;
    my $max_ldev_per_cu = 256;
    my $cu = int($devnum / $max_ldev_per_cu);
    my $hex_cu = sprintf("%x", $cu);
    return $hex_cu;
}

# Compute and return LDEV id in hex.
sub
_get_ldev_id()
{
    my $devnum = shift;
    my $max_ldev_per_cu = 256;
    my $id = $devnum % $max_ldev_per_cu;
    my $hex_id = sprintf("%x", $id);
    return $hex_id;
}

sub
get_enclosure_vdid
{
    my $self = shift;
    my $vendor = shift;
    my $product = shift;
    my $cabsno = shift;
    my $vdid;
    my $model;
    my $isHP = 0;
    if ($product =~ /^XP/)
    {
        $isHP = 1;
        $vdid = "HP_";
    }
    elsif($product =~ /SUN/)
    {
        $vdid = "SUN_";
    }
    else
    {
        $vdid = "HITACHI_";
    }

    if ($isHP == 1)
    {
        $model = $product;
        #HBAAPP uses XP12000, even if the model is 501 which is the HP XP10000, so put XP12000 in the vdid
        #do the same for XP20000 as well
        $model = "XP12000" if ($model eq "XP10000" || $model eq "XP20000");
        $vdid .= $model . "_";
      
    }
    else
    {
         #For USP or AMS/WMS we use the family
         $product =~ s/HDS//g;
         if ($product =~ /USP/ || $product =~ /R600/ || $product =~ /AMS/ )
         {
             $vdid .= $product . "_";
         }
         elsif($product =~ /WMS/)
         {
             # Making the UID same as AMS, for in band correlation
             $vdid .= "AMS_";
         }
         elsif($product =~ /D800/)
         {
             # Making the UID same as AMS, for in band correlation
             $vdid .= "AMS_";
         }
         else
         {
             # This is just temp arrangement. Need to see if anything new gets added.
             $vdid .= $product . "_";
         }
    }
    my $sno = _get_serial_no($product,$cabsno);
    return undef if ($sno eq "");
    $vdid .= $sno;
    return $vdid;
}

sub
_get_serial_no
{
    my $product = shift;
    my $sno = shift;

    if ($product =~ m/^95/)
    {
        #It is a 9500V model, so strip the serialnumber of 6xxx and use just last 4 bytes from the 8 byte serialnumber.
        $sno = substr $sno,4,4;
    }
    # Now Strip off any leading zeroes.
    $sno =~ s/^0+//; 
    return $sno;
}




1;
