#$Id: Parser.pm,v 1.1 2012/09/25 22:32:59 joe Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::SIPlugin::Parser;
use strict;
use Carp;
require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT = qw(new initialize get_object get_object_meta);
$VERSION = '3.0';

# Description: Instantiate the Parser object
sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

# Description: Load the specified parser plugin
# Input : 
#       type - the type of the parser(plugin name)
# Dies on error to load the module
sub initialize 
{
    my $self = shift;
    my $type = shift;

    my $module_name = "VRTS::Enclosure::SIPlugin::Parser::".$type;
    if (eval "require $module_name")
    {
        my $type_handle = &{\&{"${module_name}::initialize"}};
        $self->{PLUGIN} = $type_handle;
    }
    else
    {
        croak "$@ $type not supported\n";
    }
}

# Description :Contructs a hash for the specified object from the given stream
# Input :
#       self - reference to the parser object.
#       obj - Name of the object. the object definition is known to the parser plugin.
#       fh -  handle to a file that contains the o/p to be parsed.
#             the o/p is excepted to contain objects of the specified type only
#       mkstart - Marker field that identifies the beginning of a new object in the stream.
#       mkvalue - Used to read a specific object from the o/p. 
#                 if specified will read only the object whoes mkstart field matches the given value.
# Output :
#       obj - reference to the constructed hash. 
#             If no objects are found then this will be undef and ret code will be 0.
#       ret_code - 0 on success. non-zero on failure to parse(e.g. If the command that fetched the o/p failed)
sub get_object
{
    my $self = shift;
    my $obj = shift;
    my $fh = shift;
    my $mkstart = shift;
    my $mkvalue = shift;
    
    if (not defined $self->{PLUGIN})
    {
        print STDERR "Plugin not loaded\n";
        return undef;
    }
    my ($obj, $ret_code) = $self->{PLUGIN}->get_object($obj, $fh, $mkstart, $mkvalue);
    return ($obj, $ret_code);
}

# Description : Returns the object definitions known to the Parser.
# Input :
#       self - reference to the parser object.
#       obj - Name of the object whoes definition is required.
#             if not specified then all object definitions are returned.
# Output :
#       meta - reference to a hash that contains the object definitions.
#              undef if the definition(s) is not found.
sub get_object_meta
{
    my $self = shift;
    my $obj = shift;

    if (not defined $self->{PLUGIN})
    {
        print STDERR "Plugin not loaded\n";
        return undef;
    }
    my $meta = $self->{PLUGIN}->get_object_meta($obj);
    return $meta;
}
1;
