#!/opt/VRTSsfmh/bin/perl
#$Id: disable.pl,v 1.1 2012/09/25 22:32:59 joe Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

use strict;
use VRTS::Paths;
use File::Spec;
use VRTS::Util;
use VRTS::Ini;
use File::Temp;
use JSON;
use VRTS::Logger;
use VRTS::Schedule;

my $context = $ENV{context};
if (!defined $context)
{
    $context = "SI";
}

my $installdir = VRTS::Paths::get_path ("InstallDir");
my $app_dir = VRTS::Paths::get_path ("AppDir");
my $appdircs = $app_dir;
   $appdircs =~ s/VRTSsfmh/VRTSsfmcs/;
my $app_dir = VRTS::Paths::get_path ("AppDir");
my $addon_dir = $ENV{Store};

my $perl = File::Spec->catfile ($installdir, "bin", "perl");
my $array_driver = File::Spec->catfile ($installdir, "bin", "array_driver.pl");

my $log_dir = File::Spec->catdir($app_dir,"logs");
my $log = new VRTS::Logger(Module => 'siplugin',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'siplugin-setup.log',
                           UmiString => '');

if ($context eq "sdktest")
{
   exit (0);
}

my $plugin_configs;
my $vendor;
my $model;
my $array_family;
my $version;
my $addon_id;
my $ha;
my $active_cms = 0;
my $config_data;

my $ret = get_env();
goto err if ($ret);

VRTS::Logger::log ('debug', "Disable called for Plug-in $array_family");

if ($active_cms) 
{
     foreach my $encl_config (@$plugin_configs)
     {
         my $enc_id = $encl_config->{VENDOR_ID};
         my $schedid = $enc_id;

         if (defined $enc_id && $enc_id ne "")
         {
             VRTS::Schedule::pause_resume_sched($schedid, "pause");
             VRTS::Logger::log ('debug', "paused schedule for $enc_id");
         }
     }
}
else
{
   # This is CMS HA setup and disable is getting called on the standby node.
   # Nothing to disable on slave
}

VRTS::Logger::log('debug' , "Successfully disabled SI plug-in for $array_family($version)\n");
print "Successfully disabled SI plugin for $array_family($version)\n";
exit(0);

err:
    VRTS::Logger::log('error' , "Failed to disable SI Plugin for $array_family($version)\n");
    print "Failed to disable SI Plugin for $array_family($version)\n";
    exit(1);


sub get_env
{
    my $msg;

    VRTS::Logger::log ('debug', "addon-dir is $addon_dir");
    if ($context ne "sdktest")
    {
        my $cms_info = VRTS::Util::get_cms_config_info();
        if ($cms_info->{'cs'} == 0)
        {
            $msg = "Current host is not a Management Server.\n".
                   "SI Plug-in should be installed only on a Management Server\n";
            print STDERR $msg;
            VRTS::Logger::log ('error', $msg);
            return 1;
        }

        $ha = $cms_info->{'ha'};
        if ($ha)
        {
            if ($cms_info->{'active_node'} == 1)
            {
               # this is a CMS HA active node
               $active_cms = 1;
            }
        }
        else
        {
            $active_cms = 1;
        }
        VRTS::Logger::log('debug', "active_cms = $active_cms. ha = $ha");
    }

    my $config_file = File::Spec->catfile ($addon_dir, "plugin_def.conf");
    if (open (H, $config_file) == 0)
    {
        $msg = "failed to open file $config_file : $@";
        VRTS::Logger::log ('error', $msg);
    }
    else
    {
        my $buf = do { local $/; <H> };
        close (H);
        $config_data;
        eval { $config_data = JSON::jsonToObj($buf)};
        if ($@)
        {
            $msg = "Invalid config file $config_file : $@\n";
            VRTS::Logger::log ('error', $msg);
            return 1;
        }
        else
        {
           $vendor = $config_data->{VENDOR};
           $model = $config_data->{MODEL};
           $array_family = $config_data->{ARRAY_PLUGIN_FAMILY};
           $version = $config_data->{ARRAY_PLUGIN_VERSION};
           $addon_id = $config_data->{PLUGIN_ADDON_ID};

           if (!defined $vendor || $vendor eq ""
               || !defined $model || $model eq ""
               || !defined $array_family || $array_family eq ""
               || !defined $version || $version eq "")
           {
               VRTS::Logger::log('error', "Invalid plugin config file. Missing vital information");
               return 1;
           }
           $version =~ s/^v//;

           VRTS::Logger::log('debug', "Vendor [$vendor].Model [$model].".
                   "Family [$array_family].Plugin vesion [$version].Addonid $addon_id");
        }
    }
    my $inc_agentlet = File::Spec->catfile($installdir, "agentlet");
    push @INC, $inc_agentlet;

    if(eval "require VRTS::Enclosure::SIPlugin::Util")
    {
        if ($active_cms)
        {
            $plugin_configs = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_all_plugin_configs"}}($array_family);
        }
    }
    else
    {
        VRTS::Logger::log('error', "Failed to load module:$@.");
        print STDERR "Plug-in not installed completely\n".
                     "Please re-install the plug-in\n";
        return 1;
    }

    return 0;
}
