#$Id: check_env.pl,v 1.1 2012/09/25 22:32:59 joe Exp $
#
# VOM Storage Insight SDK Check Environment Script for
# the DELL Compellent Array
#
#Requires:
# Windows PowerShell 1.0 or 2.0, which require .Net Framework 2.0
# Storage Center 4.1 or higher

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

use VRTS::OS;
use Getopt::Long;

if ($^O !~ /Win32/i) {
    print STDERR "Must run in Windows with PowerShell installed.\n";
    exit 1;
}

# See if DELL Compellent PowerShell StorageCenter SnapIn is installed
my $cver1 = VRTS::OS::querykey("SOFTWARE\\Microsoft\\PowerShell\\1\\PowerShellSnapIns\\Compellent.StorageCenter.Scripting", Version);

if (! $cver1)
{
    print STDERR "DELL Compellent PowerShell StorageCenter SnapIn is not installed.\n";
    print STDERR "Version 4.1 or higher is required.\n";
    exit 2;
}

if ($cver1 < "4.1") {
    print STDERR "Need DELL Compellent PowerShell StorageCenter Snapin version 4.1 or higher\n";
    print STDERR "The current installed version is $cver1\n";
    exit 3;
}

$cmd = "powershell Set-ExecutionPolicy RemoteSigned";
system($cmd);

$scriptdir = $ENV{'SI_PLUGIN_DIR'};
$script = "$scriptdir/dellcompcli.ps1";
if (-e $script) {
    GetOptions('ip=s' => \$ip, 'user=s' => \$user, 'password=s' => \$password);
    $cmd = "powershell $script --connect --ip $ip --user $user --password $password";
    system($cmd);
    $ret = $?;
    if ($ret != 0) {
        exit $ret;
    }
}

exit 0;
