#$Id: dellcompcli.ps1,v 1.2 2013/08/01 20:05:06 joe Exp $
#
# VOM Storage Insight SDK Script For The DELL Compellent Array
#
# Must:  Set-ExecutionPolicy RemoteSigned

$VERSION = "1.2"
$TMPWORK = "c:\temp\vomsisdk"

$global:TMPD = $null
$global:storagecenterCSV = $null
$global:portsCSV = $null
$global:disksCSV = $null
$global:volumesCSV = $null
$global:volmapCSV = $null
$global:storageprofileCSV = $null
$global:cmmCSV = $null
$global:replaysCSV = $null
$global:asreplCSV = $null

function init
{
    $TD = $env:SI_PLUGIN_WORK_DIR
    if (! $TD) {
        $TD = $TMPWORK
    }

    $global:TMPD = $TD
    $global:storagecenterCSV = "$TMPD\storagecenter.csv"
    $global:portsCSV = "$TMPD\ports.csv"
    $global:disksCSV = "$TMPD\disks.csv"
    $global:volumesCSV = "$TMPD\volumes.csv"
    $global:volmapCSV = "$TMPD\volmap.csv"
    $global:storageprofileCSV = "$TMPD\storageprofile.csv"
    $global:cmmCSV = "$TMPD\cmm.csv"
    $global:replaysCSV = "$TMPD\replays.csv"
    $global:asreplCSV = "$TMPD\asrepl.csv"

    $null = mkdir -p $TD 2>&1

    $null = Add-PSSnapin Compellent.StorageCenter.PSSnapin 2>&1
    $null = Add-PSSnapin Compellent.StorageCenter.Scripting 2>&1
}

function importCSV
{
    param($file)
    $csv = import-csv $file
    if (! $csv) {
        write-host "ERROR:  Cannot open file $file" 2>&1
        exit 1
    }
    return $csv
}

function toMB
{
    param($num)
    $num = $num.replace(" ", "")
    $num /= 1MB
    return $num
}

function conn
{
    param($hostname, $user, $password)

    # connect to array
    $pass = convertto-securestring $password -asplaintext -force
    $save = "Compellent_" + $hostname
    $SCConnect = Get-SCconnection -HostName $hostname -User $user -Password $pass -Save $save -Default
    if (! $SCConnect) {
        return 1
    }
    return 0
}

function enclosureid
{
    param($sysname)
    $i = $sysname.lastindexOf(" ") + 1;
    if ($i -gt 0) {
        $eid = $sysname.substring($i);
        if ($eid) {
            $ieid = [int]$eid
            #format to hex with leading zeroes for 8 digits
            return "{0:X8}" -f $ieid
        }
    }
    return $sysname.replace(" ", "")
}

#"Index","Name","ManagementIPAddress","OperationMode","Controllers",
#"CreateTime", "CreateUser","ModifyTime","ModifyUser"
function enclosure
{
    get-scstoragecenter | export-csv -path $storagecenterCSV
    $system = importCSV $storagecenterCSV

    $eid = enclosureid $($system.Name)
    $ids = "COMPELNT_CompellentVol_" + $eid
    write-host "Id:  $ids"
    write-host "Name:  $($system.Name)"
    write-host "IPAddress:  $($system.ManagementIPAddress)"
    write-host "Vendor:  DELL"
    write-host "Product:  COMPELLENT"
    write-host "Model:  $($system.Name)"
    write-host "AutoComputeCaps:  1"
    write-host ""
}

#"Index","Name","ProfileType","WritableTier1","WritableTier2","WritableTier3",
#"ReplayTier1","ReplayTier2","ReplayTier3","CreateTime","CreateUser","ModifyTime","ModifyUser"
function volprofile
{
    param($scprofile, $profname)
    $scp = $scprofile | ? {([string]$_.Name) -eq $profname}
    return $scp
}

function volsno
{
    param($volumes, $volindex)
    $vol = $volumes | ? {([string]$_.Index) -eq $volindex}
    if ($vol) {
       return $vol.SerialNumber
    }

    return "-"
}

function isassigned
{
    param($volmap, $volindex)
    $vol = $volmap | ? {([string]$_.VolumeIndex) -eq $volindex}
    if ($vol) {
        return 1
    }
    return 0
}

#"Index","ControllerWWN","ServerWWN","ServerIndex","ServerName","LUN",
#"Status","VolumeIndex","VolumeName","ReadOnly","ActiveController",
#"CreateTime","CreateUser","ModifyTime","ModifyUser"
function volhostmap
{
    $volumes = importCSV $volumesCSV
    $volmap = importCSV $volmapCSV

    foreach($vol in $volmap) {
        $sno = volsno $volumes $($vol.VolumeIndex)
        if ($sno -eq "-") {
            continue
        }
        $id = "COMPELNT_CompellentVol_" + $sno
        write-host "Id:  $id"
        #write-host "Name:  $($vol.VolumeName)"
        write-host "HbaPortWWPN:  $($vol.ServerWWN)"
        write-host "ArrayPortWWPN:  $($vol.ControllerWWN)"
        $hexlunid = [Convert]::ToString($($vol.LUN), 16).toUpper()
        write-host "LunId:  $hexlunid"
        write-host ""
    }
}

#"Name","Index","Notes","ParentFolder","LogicalPath","Size","BlockCount",
#"SerialNumber","Status","StorageType","ReplayProfiles","StorageProfileName",
#"ActiveController","IsReplicated","IsCopying","IsMirrored","WriteCacheEnabled",
#"ReadCacheEnabled","ReadAheadCacheEnabled","ReadRequests","ReadBlocks",
#"WriteRequests","WriteBlocks","CreateTime","CreateUser","ModifyTime","ModifyUser"
function volumes
{
    $volumes = importCSV $volumesCSV
    $volmap = importCSV $volmapCSV
    $scp = importCSV $storageprofileCSV

    foreach($vol in $volumes) {
        #$profile = volprofile $scp $($vol.StorageProfileName)
        $id = "COMPELNT_CompellentVol_" + $($vol.SerialNumber)
        write-host "Id:  $id"
        write-host "Name:  $($vol.Name)"
        write-host "Thin:  1"
        $size = toMB $($vol.Size)
        write-host "LogicalSize:  $size"
        write-host "RawSize:  -"
        write-host "ThinConsumedSize:  -"
        write-host "ThinConsumedRawSize:  -"
        write-host "RaidLevel:  RAID 10"
        write-host "Layout:  -"
        $assigned = isassigned $volmap $($vol.Index)
        write-host "Assigned:  $assigned"
        write-host "Virtual:  0"
        write-host "Administrative:  0"
        $ftol = 0
        if ($($vol.isMirrored) -eq "True") {
           $ftol = 1
        }
        elseif ($($vol.StorageType).contains("Redundant")) {
           $ftol = 1
        }
        write-host "FaultTolerant:  $ftol"
        write-host "Meta:  0"
        write-host ""
    }
}

#"Position","Index","Status","Health","ControlType","DiskFolder","LogicalPath",
#"Classification","Notes","Enclosure","Capacity","FreeSpace","TotalBlocks",
#"UnallocatedBlocks","Vendor","Product","SerialNumber","CreateTime",
#"CreateUser","ModifyTime","ModifyUser"
function disks
{
    $disks = importCSV $disksCSV

    foreach($disk in $disks) {
        $id = "COMPELNT_CompellentVol_" + $($disk.SerialNumber)
        write-host "Id:  $id"
        write-host "Name:  $($disk.Position)"
        $capacity = toMB $($disk.Capacity)
        write-host "PhysicalSize:  $capacity"
        $freespace = toMB $($disk.FreeSpace)
        write-host "AvailableSize:  $freespace"

        $state = 1
        if ($($disk.Health) -eq "Healthy") {
           $state = 0
        }
        write-host "State:  $state"

        if ($($disk.ControlType) -eq "ManagedSpare") {
           write-host "SpareSize:  $capacity"
        }
        else {
           write-host "SpareSize:  0"
        }

        write-host "Shared:  0"
        write-host "RPM:  -"
        write-host ""
    }
}

#"Index","ControllerWWN","ServerWWN","ServerIndex","ServerName",
#"LUN","Status","VolumeIndex","VolumeName","ReadOnly","ActiveController",
#"CreateTime","CreateUser","ModifyTime","ModifyUser"
function adapters
{
    $ports = importCSV $portsCSV
    $system = importCSV $storagecenterCSV

    $adapters = @()
    foreach($port in $ports) {
       $ad = $($port.OriginalController)
       if ($adapters -notcontains $ad) {
          $adapters += $ad
          $sysname = $($system.Name).replace(" ", "")
          $id = "COMPELNT_CompellentVol_" + $sysname + "_" + $ad
          write-host "Id:  $id"
          write-host "Clustered:  0"
          write-host "Type:  $($port.Type)"
          #write-host "Card:  $($port.Slot)"
          write-host "Name:  $ad"  #$($port.Description)"
          write-host "Mode:  1"
          write-host ""
       }
    }
}

#"Index","Controller","OriginalController","Slot","Port",
#"WorldWideName","Type","Description","Status","MapCount",
#"IPAddress","IQN","FaultDomain","CreateTime","CreateUser",
#"ModifyTime","ModifyUser"
#"9","5402","5402","5","3","5000D31000151A07","FC","QLogic QLE2464 4G Fibre Channel Adapter","Up","204","0.0.0.0","","2","2/9/2012 9:41:52 AM","","3/13/2012 5:41:56 PM","Admin"
function ports
{
    $ports = importCSV $portsCSV
    $system = importCSV $storagecenterCSV

    $sysname = $($system.Name).replace(" ", "")

    foreach($port in $ports) {
        write-host "Id:  $($port.WorldWideName)"
        write-host "Name:  $($port.Port)"
        write-host "Type:  $($port.Type)"
        $status = 1
        if ($($port.Status) -eq "Up") {
            $status = 0
        }
        write-host "Status:  $status"
        write-host "WWPN:  $($port.WorldWideName)"  # world wide port number of the port
        write-host "WWNN:  $($port.WorldWideName)"  # world wide node number of the port node
        $controller = $($port.Controller)
        $controller = "COMPELNT_CompellentVol_" + $sysname + "_" + $controller
        write-host "AdapterId:  $controller"
        write-host ""
    }
}

#"Name","Index","Notes","SourceVolumeIndex","SourceVolumeName","CMMIndex","CMMType","CMMState","ReplicationQosIndex","ReplicationQosName","RemoteSystemIndex","RemoteSystemName","RemoteVolumeIndex","ExternalDiskIndex","ReplicateActiveReplay","Deduplication","UsingLiveVolume","StorageCenterSerialNumber","CreateTime","CreateUser","ModifyTime","ModifyUser"
function asyncreplications
{
    $asreplfile = Get-Item $asreplCSV
    if ($asreplfile.length -eq 0) {
        return
    }

    $asrepl = importCSV $asreplCSV
    $volumes = importCSV $volumesCSV

    foreach($repl in $asrepl) {
        $sno = volsno $volumes $($repl.SourceVolumeIndex)
        if ($sno -eq "-") {
           continue
        }

        $id = "COMPELNT_CompellentVol_" + $sno
        write-host "Id: $id"
        write-host "RemoteEnclName: $($repl.RemoteSystemName)"


        # Have parter index(RemoteVolumeIndex), but not name        
        #write-host "PartnerId:
        #write-host "PartnerName: 

        # 0 = source. 1 = target
        $d = 0
        write-host "Designation:  $d"
        write-host "AcrossEncls:  1"
        write-host "RepMode:  Async"
        write-host "RepType:  $($repl.CMMType)"

        $state = "Synchronized"
        if ($($repl.CMMState) -eq "Running") {
            $state = "Synchronizing"
        }
        write-host "RepState:  $state"
        write-host "AccessMode:  Rdwrite"
        write-host ""
    }
}

#"Index","Type","State","Priority","DestinationVolumeIndex",
#"DestinationVolumeName","SourceVolumeIndex","SourceVolumeName",
#"CopyReplayHistory","CurrentReplayIndex","CurrentPosition",
#"TotalCopySize","ActiveController","CreateTime","CreateUser",
#"ModifyTime","ModifyUser"
function replications
{
    $cmmfile = Get-Item $cmmCSV
    if ($cmmfile.length -eq 0) {
        return
    }

    $cmm = importCSV $cmmCSV
    $volumes = importCSV $volumesCSV
    $replays = importCSV $replaysCSV

    foreach($repl in $cmm) {
        $psno = volsno $volumes $($repl.DestinationVolumeIndex)
        if ($psno -eq "-") {
           continue
        }

        $sno = volsno $volumes $($repl.SourceVolumeIndex)
        if ($sno -eq "-") {
           continue
        }

        $id = "COMPELNT_CompellentVol_" + $sno
        write-host "Id: $id"
        $id = "COMPELNT_CompellentVol_" + $psno
        write-host "PartnerId:  $id"
        write-host "PartnerName:  $($repl.DestinationVolumeName)"

        # 0 = source. 1 = target
        $d = 0
        write-host "Designation:  $d"
        write-host "AcrossEncls:  0"
        write-host "RepMode:  Sync"
        write-host "RepType:  $($repl.Type)"
        $state = "Synchronized"
        if ($($repl.State) -eq "Running") {
            $state = "Synchronizing"
        }
        write-host "RepState:  $state"
        write-host "AccessMode:  Rdwrite"
        write-host ""
    }
}

function getdata
{
    get-scport | export-csv -path $portsCSV
    get-scdisk | export-csv -path $disksCSV
    get-scvolume | export-csv -path $volumesCSV
    get-scvolumemap | export-csv -path $volmapCSV
    get-scstorageprofile | export-csv -path $storageprofileCSV
    get-screplay | export-csv -path $replaysCSV
    get-sccmm | export-csv -path $cmmCSV
    get-scasyncreplication | export-csv -path $asreplCSV

#   get-scstoragecenter | export-csv -path $storagecenterCSV
    $null = Move-Item $TMPWORK/storagecenter.csv $TMPD 2>&1
}

function cleanup
{
    #remove-item $TMPD -recurse
}

function usage
{
  write-host "VOM Storage Insight SDK Script For The DELL Compellent Array:  version $VERSION"
  write-host "        dellcompcli.ps1 --list encls [ --ip <ip_addr> --user <username> --password <password> ]"
  write-host "        dellcompcli.ps1 --list {pdevs | ldevs | ports | adapters | ldevhostmap | replications}"
  write-host "        dellcompcli.ps1 {--get-data | --cleanup}"
  exit 1
}

##################################
$i=0
foreach ($arg in $args) {
    if ($arg -eq "--get-data") {
        $op = "get-data"
        break
    } elseif ($arg -eq "--connect") {
        $op = "connect"
    } elseif ($arg -eq "--list") {
        $op = $args[$i+1]
        if ($op -ne "encls") {
            break
        }
    } elseif ($arg -eq "--cleanup") {
        $op = "cleanup"
        break
    } elseif ($arg -eq "--ip") {
        $ip = $args[$i+1]
    } elseif ($arg -eq "--user") {
        $user = $args[$i+1]
    } elseif ($arg -eq "--password") {
        $password = $args[$i+1]
    }
    elseif ($arg.startswith("-")) {
        usage
    }
    $i++
}

init
if ($op -eq "encls" -or $op -eq "connect") {
    if ($ip -and $user -and $password) {
       $rc = conn $ip $user $password
       if ($rc -eq 1) {
           write-host "ERROR:  Connection to array $ip failed." 2>&1
           exit 1
       }
    }
    if ($op -eq "connect") {
        exit 0
    }
}

if ($op -eq "encls") {
    enclosure
} elseif ($op -eq "pdevs")  {
    disks
} elseif ($op -eq "ldevs") {
    volumes
} elseif ($op -eq "ports") {
    ports
} elseif ($op -eq "adapters") {
    adapters
} elseif ($op -eq "ldevhostmap") {
    volhostmap
} elseif ($op -eq "replications") {
    replications
    asyncreplications
} elseif ($op -eq "get-data") {
    getdata
} elseif ($op -eq "cleanup") {
    cleanup
} else {
    usage
}

exit 0
