#!/opt/VRTSsfmh/bin/perl

#$Id: sisdk_ui_getDeviceIdentifiers.pl,v 1.1 2012/09/25 22:33:00 joe Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use lib;
use strict;
use CGI;
use JSON;
use File::Spec;
use VRTS::Paths;
use File::Spec;
use File::Path;
use File::Copy;
use File::Temp;
use VRTS::Util;
use POSIX;

use IO::Handle;
my $appdir = VRTS::Paths::get_path("AppDir");
my $log_dir = File::Spec->catdir($appdir,"logs");
my $installdir  = VRTS::Paths::get_path ("InstallDir");

main();

sub main{
    my $query = new CGI();
    my $log = new VRTS::Logger(Module => 'sisdk', Logdir => $log_dir, Level => 'debug', Logfile => 'sisdkui.log', UmiString => '');

    print "Content-type:text/html\r\n\r\n";
    print getDeviceIdentifierTableHeader($query);
    my $deviceIdentifer = $query->cookie('deviceIdentifer');
   
    VRTS::Logger::log('debug', "<deviceIdentifer>". $deviceIdentifer);
 
    if ($deviceIdentifer) {    
        my $device_hash;
        eval {
            $device_hash = JSON::jsonToObj($deviceIdentifer);
        };
        foreach my $key (sort keys %{$device_hash})
        {
            my $format = $device_hash->{$key}->{"format"};
            my $startByte = $device_hash->{$key}->{"start_byte"};
            my $pgno = $device_hash->{$key}->{"pgno"};
            my $noOfBytes = $device_hash->{$key}->{"no_bytes"};
            my $page83Id = $device_hash->{$key}->{"pg83id_type"};
            VRTS::Logger::log('debug', $key."<getDeviceIdentifier><format>".$format."<startByte>".$startByte."<pgno>". $pgno."<noOfBytes>" .$noOfBytes);
            my $numberPart = 1;
            if ($key =~ /^\$\{F([0-9]+).*/) {
                $numberPart = $1;
                #my $field ="Field" . $numberPart;
                print getDeviceIdentifierTableRows($numberPart , $format, $startByte,  $pgno, $noOfBytes, $page83Id) ;
            }
            my $javaScript = ' <SCRIPT id="toggleTableField" type="text/javascript">  updatePage83IdField(); </SCRIPT>'; 
            print $javaScript;
        }
    }else{
        my $javaScript = ' <SCRIPT id="toggleTableField" type="text/javascript"> updatePage83IdField(); </SCRIPT>'; 
        print getDeviceIdentifierTableRows(1 , "", "",  "", "", "") . $javaScript;
    }
   # print getDeviceIdentifierTableRows();
    print getDeviceIdentifierTableEnd();
    print getIdentiferFormula($query);
    #if ("" eq $deviceIdentifer) {
    #   print join "",  getConfigureHeader(), oneEmptyRow();
    #}else{
    #   print join "",  getConfigureHeader(), getConfigParameters("", $query);
    #}
}



sub getDeviceIdentifierTableHeader{
    my $query = shift;
    my $iVersionId = $query->cookie("iVendorId");
    my $iProductId = $query->cookie("iProductId");   
    if (!$iVersionId) {
      $iVersionId = $query->cookie("iVendor");
      $iProductId = $query->cookie("iModel");
    }    

    my $header ="";
       $header .= '<table width="100%" border="0" ellpadding="1" cellspacing="1"';
	   $header .= 'class="middletable" id="tbVendorTable" valign="top">';
       $header .= '  <tr>';
       $header .= '     <td width="140" class="contentpane1" style="">* SCSI Vendor Id:<span></td>';
       $header .= '     <td colspan="5"><input type="text" name="iVendorId" id="iVendorId" value ="' . $iVersionId .'"></td>';
       $header .= '   </tr>';
       $header .= '  <tr>';
       $header .= '     <td width="140" class="contentpane1" style="">* SCSI Product Id:</span></td>';
       $header .= '     <td colspan="5"><input type="text" name="iProductId" id="iProductId" value="' . $iProductId .'"></td>';
       $header .= '   </tr>';
       $header .= '</table>';
       $header .= '<table width="100%" border="0" ellpadding="1" cellspacing="1"';
	   $header .= 'class="middletable" id="tbDeviceIdTable" valign="top">';
	   $header .= '<tbody id="tbodyDeviceId">';
	   #$header .= '<tr class="component_title">';
	   #$header .= '<td height="25" class="grid_td" colspan="6"><b>Specify Formula to be used for constructing Device Identifier(s)</b></td>';
	   #$header .= '</tr>';

 	   $header .= '<tr>';
  	   $header .= '  <td height="10" colspan="6"></td>';
	   $header .= '</tr>';
	   $header .= '<tr id="trDeviceTableHeader">';
	   $header .= '   <td width="70" class="contentpane1" style="">Field Name</td>';
 	   $header .= '   <td width="100" class="contentpane1" style="">SCSI Page No.</td>';
	   $header .= '   <td width="70" class="contentpane1"  style="">Start Byte</td>';
	   $header .= '   <td width="70" class="contentpane1" style="">No of Byte(s)</td>';
	   $header .= '   <td width="120" class="contentpane1" style="">Page 83 Id Type</td>';
	   $header .= '   <td width="100" class="contentpane1" style="">Field Format</td>';
	   $header .= '   <td width="60">&nbsp;</td>';
	   $header .= '</tr>';
    return $header;
}

sub getDeviceIdentifierTableEnd(){
    my $end ="";
       $end .='</tbody>';
       $end .='</table>';
    return $end;
}

sub getDeviceIdentifierTableRows{
     my $numberPart = shift;
     my $format = shift;
     my $startByte = shift;
     my $dpgno = shift;
     my $noOfBytes = shift;
     my $page83Id = shift;
     my $row = "";
        $row .= ' <tr id="rowCount_'.$numberPart.'">';
		$row .='<td width="70"><input type="text" name="Field_'.$numberPart.'"';
		$row .='      class="inputbox" id="Field_'.$numberPart.'" size="10" readonly ="readonly" value="Field' . $numberPart . '"></span>';
		$row .='    </td>';
		$row .='<td width="100"><select name="sePageNoSelect_'.$numberPart.'"';
		$row .='     width="95" style="width: 95px" onchange="togglePage83IdField(this, ' . $numberPart .')" id="sePageNoSelect_'.$numberPart.'">';

		my $pgno = sprintf("0x%02x", $dpgno);        
        if ($pgno eq "0x83" ) {
          $row .='    <option value="0x83" selected="selected">0x83</option>';
        }else{
          $row .='    <option value="0x83">0x83</option>';
        }
        if ($pgno eq "0x80" ) {
          $row .='    <option value="0x80" selected="selected">0x80</option>';
        }else{
          $row .='    <option value="0x80">0x80</option>';
        }
        if ($pgno eq "0x00" ) {
          $row .='    <option value="0x00" selected="selected">0x00</option>';
        }else{
          $row .='    <option value="0x00">0x00</option>';
        }

		$row .='     </select></td>';
        $row .='<td width="100"><input type="text" name="StartByteField_'.$numberPart.'"';
		$row .='     class="inputbox" size="9" id="StartByteField_'.$numberPart.'" value="'. $startByte . '" onblur ="checkStartByteData(this)"></td>';
		$row .='<td width="50"><input type="text"';
		$row .='     name="NofByteField_'.$numberPart.'"';
		$row .='     class="inputbox" size="9" id="NofByteField_'.$numberPart.'" value="'. $noOfBytes . '" onblur ="checkNofByteData(this)" ></td>';

        if ($pgno eq "0x83" ) {
		    $row .='<td width="50"><input type="text" ';
		}else{
            $row .='<td width="50"><input type="text" disabled ="disabled"  ';
        }
        $row .='     name="Page83IdField_'.$numberPart.'"';
		$row .='     class="inputbox" size="13" id="Page83IdField_'.$numberPart.'" value="'. $page83Id . '" onblur ="checkPage83IdData(this)" ></td>';

		$row .='<td width="60"><select name="FieldFormatSelect_'.$numberPart.'"';
		$row .='     class="listmenu1" id="FieldFormatSelect_'.$numberPart.'">';
        if ($format eq 0) {        
		    $row .='     <option value="0" selected="selected">Binary</option>';
        }else{
            $row .='     <option value="0">Binary</option>';
        }
        if ($format eq 1) {        
    		$row .='     <option value="1" selected="selected">Integer</option>';
        }else{
            $row .='     <option value="1">Integer</option>';
        }
        if ($format eq 2) {        
            $row .='     <option value="2" selected="selected">Hex</option>';
        }else{
            $row .='     <option value="2">Hex</option>';
        }
		$row .='     </select></td>';
        if ($numberPart eq 1 ) {
            $row .='<td width="60" nowrap="nowrap"><a href="javascript:void(0);"';
            $row .='     id="HrAddDirSynchPair1" onclick="addDeviceIdentiferRow(\'tbDeviceIdTable\')">';
            $row .='        <img border="0" alt="Add row" id="ImDirSynchAdd1"';
            $row .='         src="images/add_tab.gif" title="Add row"></a></td>';
        }else{
            $row .=' <td><div><a href="javascript:void(0);" onclick="deleteDeviceIdentiferRow(&quot;rowCount_'. $numberPart .'&quot;)" id="HrDelRow_'.$numberPart .'">';
            $row .=' <img border="0" id="ImDeleteRow_' .$numberPart.'" alt="Delete row" src="images/subtract_tab.gif" title="Delete row"></a></div></td>';
        }
        $row .='</tr>';

    return  $row ;
}
sub getIdentiferFormula {
   my $query=shift;
   my $encIdentiferFormula = $query->cookie('EncIdentiferFormula');
   my $logicalIdentiferFormula = $query->cookie('LogicalIdentiferFormula');

   my $fomula ='<table ellpadding="1" id="tbIdentifierFormula" border="0" cellspacing="1"';
	  $fomula .=' class="middletable">';
	  $fomula .='<tr>';
	  $fomula .='<td height="2" colspan="2"></td>';
	  $fomula .='</tr>';
	  $fomula .='<tr>';
	  $fomula .='<td height="25" class="contentpane1">Enclosure Identifer Formula:</td>';
	  $fomula .=' <td><input type="text" size="55" name="EncIdentiferFormula" class="inputbox" id="EncIdentiferFormula" value="'. $encIdentiferFormula.'"></td>';
      $fomula .='</tr>';
      if ($query->cookie('chDiscoverLogicalDisk')) {
        $fomula .='<tr>';
	    $fomula .='<td height="25" class="contentpane1">Logical Disk Identifer Formula:</td>';
	    $fomula .='<td><input type="text" size="55" name="LogicalIdentiferFormula" class="inputbox" id="LogicalIdentiferFormula" value="'. $logicalIdentiferFormula.'"></td>';
	    $fomula .='</tr>';            
        $fomula .='</table>';
      }
     return $fomula;
}



