#!/opt/VRTSsfmh/bin/perl
#$Id: sisdk_ui_getSummaryData.pl,v 1.1 2012/09/25 22:33:00 joe Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use lib;
use strict;
use CGI;
use JSON;
require VRTS::Paths;
use File::Path;
use File::Spec;
use IO::Handle;

my $appdir =  VRTS::Paths::get_path ("AppDir");

main();

sub main{
    my $query = new CGI();
    #my $updatedConfig = updateConfiData($OriginalConfig, $query);
    print "Content-type: text/html\n\n";
    printPluginInformation($appdir, $query);

    my $physicalBody = physicalFeatureTableBody($appdir, $query);
    my $logicalDisplay = logicalFeatureBody($appdir, $query);	
    
    if (length($physicalBody)>10 || length ($logicalDisplay)> 10)  {
       print  featureTitle();
    }
    if (length($physicalBody) > 10) {
       print physicalFeatureTable();
       print $physicalBody;
       print physicalFeatureTableEnd();
    }
    if (length ($logicalDisplay)> 10) {
       print logicalFeatureTableStart();
       print $logicalDisplay;
       print logicalFeatureTableEnd();
    }
    if ($query->cookie('EncIdentiferFormula')) {    
       print printDeviceIdentifer($appdir, $query);
    }
    my $paramDisplay = configureParamTableBody($appdir, $query);
    if (length($paramDisplay	) > 10) {
        configureParamTableStart();
        print $paramDisplay;
        configureParamTableEnd();
    }
}

sub printPluginInformation(){
    my ($OriginalConfig, $query) = @_;
    my $iName = $query->cookie('iName');
    my $iVersion = $query->cookie('iVersion');
    if ($iVersion) {
       $iVersion =~ s/^.//;
    }
    my $iFamily = $query->cookie('iFamily');
    my $iVendor = $query->cookie('iVendor');
    my $iModel = $query->cookie('iModel');

    my $commandScript = $query->param("uploadedCommandDirectory");


print <<PLUGIN_HTML;
    <table width="610" border="0" align="center">
        <tr class="component_title">                
            <td class="" align="left"><b>Plug-in Summary</b></td>
        </tr> 
    </table>
    <table border="0" width="610"  align="center">
        <tr> 
            <td  colspan="2"></td>
        </tr>
        <tr>
            <td width="120">SI Plug-in Name:</td>
            <td><span id="pluginName"><strong> $iName </strong><span></td>
        </tr>
        <tr>
            <td width="120">SI Plug-in Version:</td>
            <td><span id="pluginVersion"><strong>$iVersion</strong></span></td>
        </tr>
        <tr>
            <td width="120" >Enclosure Vendor:</td>
            <td><span id="enclosureVendor"><strong>$iVendor</strong></span></td>
        </tr>
        <tr>
            <td width="120" >Enclosure Product:</td>
            <td><span id="enclosureModel"><strong>$iModel</strong></span></td>
        </tr>

        <tr>
            <td width="120">Discovery Method:</td>
            <td ><span id="pluginFamily"><strong>$iFamily</strong></span></td>
        </tr>
    </table>
PLUGIN_HTML
}

sub printDeviceIdentifer(){
    my ($OriginalConfig, $query) = @_;
    my $identiferFormula = $query->cookie('EncIdentiferFormula');
    my $logicalIdentiferFormula = $query->cookie('LogicalIdentiferFormula');
    my $chDiscoverLogicalDisk = $query->cookie('chDiscoverLogicalDisk'); 
    my $deviceIdentifier ='<table width="610" border="0" align="center">';
       $deviceIdentifier.=' <tr>';                
       $deviceIdentifier.='  <td height="10"></td>';
       $deviceIdentifier.=' </tr> ';
       $deviceIdentifier.=' <tr class="component_title"> ';               
       $deviceIdentifier.=' <td class="" align="left"><b>Formula to be used for constructing device identifier(s)</b></td>';
       $deviceIdentifier.=' </tr> ';
       $deviceIdentifier.='</table>';
       $deviceIdentifier.='<table border="0" width="610"  align="center">';
       $deviceIdentifier.='  <tr> ';
       $deviceIdentifier.='    <td  colspan="2"></td>';
       $deviceIdentifier.='</tr>';
       $deviceIdentifier.='<tr>';
       $deviceIdentifier.=' <td width="150">Enclosure Identifier Formula:</td>';
       $deviceIdentifier.=' <td><span id="spIdentiferFormula"><strong>'. $identiferFormula.' </strong><span></td>';
       $deviceIdentifier.='</tr>';
   
    my $logicalIdentifierFormula = ' <tr>';
       $logicalIdentifierFormula .=' <td width="150">Logical Disk Identifier Formula:</td>';
       $logicalIdentifierFormula .=' <td><span id="spLogicalIdentiferFormula"><strong>'. $logicalIdentiferFormula.'</strong></span></td>';
       $logicalIdentifierFormula .='</tr>';

    if ($chDiscoverLogicalDisk) {
       return $deviceIdentifier . $logicalIdentifierFormula . '</table>';
    }else{
       return $deviceIdentifier . '</table>';
    }   
}

sub featureTitle(){
my $featureTitle =<<HEATURE_TITLE;
    <table width="610" border="0" align="center">
        <tr>
            <td class="line"></td>
        </tr>
    </table>
    <table width="610" border="0" align="center">
        <tr class="component_title">                
            <td class="" align="left"><b>Following features will be discovered via the Plug-in</b></td>
        </tr> 
    </table>
HEATURE_TITLE
  return  $featureTitle;
}

sub physicalFeatureTable(){

my $physicalFeatureTable=<<PHYSICAL_FEATURE_TABLE;

    <table width="610" border="0" align="center">
        <tr>
            <td class="" align="left"><b>Physical Characteristics</b></td>
        </tr>
    </table>
    <table width="610" border="0" align="center">
        <tr>
            <td class="line"></td>
        </tr>
    </table>
    <table id = "tbLogicalFeatures" width="100%" border="0" cellpadding="0" cellspacing="0" border="0">
        <tr>
            <td width="5"></td>
            <td>
                <table width="550" border="0" align="center" > 

PHYSICAL_FEATURE_TABLE
    return $physicalFeatureTable;
}


sub physicalFeatureTableBody(){
   my ($O, $query) = @_;

   my $chDiscoveryPhysicalDisk = $query->cookie('chDiscoveryPhysicalDisk');
   my $chDiscoveryPhysicalGroup = $query->cookie('chDiscoveryPhysicalGroup');
   my $chDiscoverArrayPort = $query->cookie('chDiscoverArrayPort');
   my $chDiscoverAdapter = $query->cookie('chDiscoverAdapter');
   my $chDiscoverRaidGroup = $query->cookie('chDiscoverRaidGroup');
   my $iPhysDisplay ="";

   my $cliCommand = $query->cookie('cliCommand');

   if ($chDiscoveryPhysicalDisk) {
        $iPhysDisplay = join "", $iPhysDisplay,  '<tr id="trPhysicalDisk">';				
        $iPhysDisplay = join "", $iPhysDisplay,  '	<td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	<td colspan="3"> Physical disks</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
   
        $iPhysDisplay = join "", $iPhysDisplay,  '<tr id="trPhysicalCommand">';				
        $iPhysDisplay = join "", $iPhysDisplay,  '	<td width="25" class="contentpane1">&nbsp;</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	<td colspan="3">Command:&nbsp;&nbsp;<b>' , $cliCommand, ' --list pdevs', '</b></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
   
   }
   if ($chDiscoveryPhysicalGroup) {
        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trPhysicalGroups">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3"> Physical groups</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
  
        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trPhysicalGroupsCommand">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, '</b></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
   }
   if ($chDiscoverArrayPort) {
        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trArrayPorts">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3"> Array ports</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';	   

        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trArrayPortsCommand">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list ports', '</b></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
   }
   if ($chDiscoverAdapter) {
        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trAdapters">';
        $iPhysDisplay = join "", $iPhysDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	<td colspan="3"> Adapters</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';

        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trAdaptersCommand">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list adapters', '</b></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
   }
   if ($chDiscoverRaidGroup) {
        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trRaidGroup">';
        $iPhysDisplay = join "", $iPhysDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	<td colspan="3">Raid group</td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';

        $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trRaidGroupCommand">';
        $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' -—list raidgroups', '</b></td>';
        $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
        
        my $chRaidGroupPhysicalMapping = $query->cookie('chRaidGroupPhysicalMapping');
        if ($chRaidGroupPhysicalMapping) {        
            $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trRaidGroup">';
            $iPhysDisplay = join "", $iPhysDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
            $iPhysDisplay = join "", $iPhysDisplay, '	<td colspan="3">Raid group to source physical disk papping</td>';
            $iPhysDisplay = join "", $iPhysDisplay, '</tr>';

            $iPhysDisplay = join "", $iPhysDisplay, '<tr id="trRaidGroupCommand">';
            $iPhysDisplay = join "", $iPhysDisplay, '  <td width="25" class="contentpane1"></td>';
            $iPhysDisplay = join "", $iPhysDisplay, '	 <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' -—list rgpdevmap', '</b></td>';
            $iPhysDisplay = join "", $iPhysDisplay, '</tr>';
        }

   }    

   return $iPhysDisplay;
}

sub physicalFeatureTableEnd(){
my $physicalFeatureTableEnd=<<PHYSICAL_FEATURE_TABLE_END;
        </table>
       </td>
      </tr>
     </table>
PHYSICAL_FEATURE_TABLE_END
  return $physicalFeatureTableEnd;
}

sub logicalFeatureTableStart(){
my $logicalTableStart=<<LOGICAL_FEATURE_START;
    <table width="610" border="0" align="center">
        <tr>
            <td class="" align="left"><b>Logical Characteristics</b></td>
        </tr>
    </table>
    <table width="610" border="0" align="center">
        <tr>
            <td class="line"></td>
        </tr>
    </table>
    <table id = "tbLogicalFeatures" width="100%" border="0" cellpadding="0" cellspacing="0" border="0">
        <tr>
            <td width="5"></td>
            <td>
            <table width="550" border="0" align="center" > 

LOGICAL_FEATURE_START
    return $logicalTableStart;
}

sub logicalFeatureBody(){
   my ($O, $query) = @_;

   my $chDiscoverLogicalDisk = $query->cookie('chDiscoverLogicalDisk');
   my $chDiscoverReplications = $query->cookie('chDiscoverReplications');
   my $chLogicalGroup = $query->cookie('chLogicalGroup');
   my $chDiscoverLogPhyMapping = $query->cookie('chDiscoverLogPhyMapping');
   my $chDiscoverLogHostMapping = $query->cookie('chDiscoverLogHostMapping'); 
   my $chRaidGroupLogicalDiskMapping = $query->cookie('chRaidGroupLogicalDiskMapping');
   my $chMetaLogicalDisk = $query->cookie('chMetaLogicalDisk');
 
   my $chDiscoverThinPools = $query->cookie('chDiscoverThinPools'); 
   my $cliCommand = $query->cookie('cliCommand');

   my $logicalDisplay = "";
   if ($chDiscoverLogicalDisk) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Logical disks</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list ldevs', '</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
  
      if ($chDiscoverLogHostMapping) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Logical disk to host mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list ldevhostmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';      
      }

     if ($chDiscoverLogPhyMapping) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Logical to physical disk mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list ldevpdevmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }
     if ($chRaidGroupLogicalDiskMapping) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Raid group to logical disk mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list rgldevmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }
     if ($chMetaLogicalDisk) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Meta logical disks</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Command:&nbsp;&nbsp;<b>', $cliCommand, '</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }
   }

   if ($chDiscoverReplications) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Replications</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' -—list replications', '</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
   }
   if ($chLogicalGroup) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Logical groups</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, '</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
   }
   if ($chDiscoverThinPools) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3">Thin pools</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3">Command:&nbsp;&nbsp;<b>', $cliCommand, ' -—list thinpools', '</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      my $chThinPoolSourceDeviceMapping = $query->cookie('chThinPoolSourceDeviceMapping');
      my $chLogicalThingPool = $query->cookie('chLogicalThingPool');
      my $chPhysicalThingPool = $query->cookie('chPhysicalThingPool');
 
      if ($chLogicalThingPool) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Thin pool to source logical disk mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list tpsrcldevmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }
      if ($chPhysicalThingPool) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Thin pool to source physical disk mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list tpsrcpdevmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }
      my $chThinPoolLogicalDiskMapping = $query->cookie('chThinPoolLogicalDiskMapping');
      if ($chThinPoolLogicalDiskMapping) {
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, ' <td width="25" class="contentpane1"><img border="0" src="images/active_light.gif"></td>';
        $logicalDisplay = join "", $logicalDisplay, ' <td colspan="3"> Thin pool to logical disk mapping</td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
        $logicalDisplay = join "", $logicalDisplay, '<tr>';
        $logicalDisplay = join "", $logicalDisplay, '  <td width="25" class="contentpane1"></td>';
        $logicalDisplay = join "", $logicalDisplay, '  <td colspan="3"> Command:&nbsp;&nbsp;<b>', $cliCommand, ' --list tpldevmap','</b></td>';
        $logicalDisplay = join "", $logicalDisplay, '</tr>';
      }

   }


   return $logicalDisplay;
}

sub logicalFeatureTableEnd(){
my $logicalTableEnd=<<LOGICAL_FEATURE_END;

            </table>
        </td>
        </tr>
    </table>
LOGICAL_FEATURE_END
}

sub configureParamTableStart(){
print <<CONFIGURE_PARAM_TABLE_START;

    <table width="610" border="0" align="center" id="tbConfigParaTitle">
        <tr>                
            <td height="10"></td>
        </tr>        
        <tr class="component_title">                
            <td class="" align="left"><b>Configuration Parameters Required to Configure a New Device</b></td>
        </tr> 
    </table>
    <table width="610" border="0" align="center" id = "tbConfigParameters">
        <tr>
            <td colspan="4" class="contentpane1"></td>
        </tr>
        <tr bgcolor="#bbd8e5" font-weight: bold>
            <td width="100" class="contentpane1">Display Name</td>
            <td width="100" class="contentpane1">CLI Option</td>
            <td width="110" class="contentpane1">Default Value</td>
            <td width="100" class="contentpane1">Parameter Type</td>
            <td width="60" class="contentpane1">Mandatory</td>
        </tr>	
CONFIGURE_PARAM_TABLE_START
}

sub configureParamTableBody(){
    my ($O, $query) = @_;
    my $configParamSets = $query->cookie('iConfigureParams');	
    my @configParamArray = split(':::', $configParamSets);
    my $size = @configParamArray;
    my $configParmCount = 0;
    my $paramDisplay ="";
    foreach my $oneSet (@configParamArray) {		
    my $configParmEntry = join "", "field", "_",  $configParmCount;
    if (length($oneSet)>3) {
    my @params = split(',,,', $oneSet);		
    my $name = @params[0];
    my $cliName = @params[1];
    my $defaultValue = @params[2];
    my $datatype = @params[3];
    my $is_mandatory = @params[4];
    if ($is_mandatory eq 0) {
        $is_mandatory = "No";
    }else{
        $is_mandatory = "Yes";
    }
    my $dataTypeDisplay = dataTypeString($datatype);
    if ($name) {
        my $rowBgColor = "";
        my $configParamRowId =  join "", "trConfigParam_", $configParmCount;
        if ($configParmCount % 2 == 1) {
            $rowBgColor = join "", '<tr bgcolor="#FFFFFF" id="', $configParamRowId, '">';
        }else{
            $rowBgColor = join "", '<tr bgcolor="#ccffff" id="', $configParamRowId, '">'; 
        }  

my $paraRow =<<PARAM_ROW;
    <td width="100" class="contentpane1">$name</td>
    <td width="100" class="contentpane1">$cliName</td>
    <td width="120">$defaultValue</td>
    <td width="80">$dataTypeDisplay</td>
    <td width="60">$is_mandatory</td>
    </tr>
PARAM_ROW
    $paramDisplay = join "", $paramDisplay, $rowBgColor, $paraRow;
    $configParmCount = $configParmCount + 1;
    }
  }
 } 
return $paramDisplay;
}

sub configureParamTableEnd(){
print <<CONFIGURE_TABLE_END;
    </table>
    <table width="610" border="0" align="center">
    <tr>
        <td class="line"></td>
    </tr>
    </table>
CONFIGURE_TABLE_END
}

sub dataTypeString(){
   my $type = $_[0];
   if ("STRING" eq $type ) {
      return "String";
   }
   elsif ( "INTEGER" eq $type ){
      return "Integer";
   }elsif ("PASSWORD_STRING" eq $type) {
      return "Password";
   }elsif ("BOOLEAN" eq $type){
      return "Boolean";
   }
}
