#!/opt/VRTSsfmh/bin/perl
#$Id: setup.pl,v 1.1 2012/10/11 19:14:41 joe Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

use strict;
use VRTS::Paths;
use File::Spec;
use VRTS::Util;
use VRTS::Logger;
use Getopt::Long;
use VRTS::AddOnInfo;
use File::Path;

my $version = "6.1.0.0";

my $app_dir = VRTS::Paths::get_path ("AppDir");
my $addon_dir = $ENV{Store};
my $installdir = VRTS::Paths::get_path ("InstallDir");

my $perl = File::Spec->catfile ($installdir, "bin", "perl");

my $log_dir = File::Spec->catdir($app_dir,"logs");
my $log = new VRTS::Logger(Module => 'sisdk',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'sisdk.log',
                           UmiString => '');
my $ret;
my $msg;
my %opts;
my $si_installed;
my $si_ver;
chdir $addon_dir;

GetOptions(\%opts,qw (operation=s));
$opts{operation} = "install" if (!defined $opts{operation});

my @install_dirs = ("$installdir/lib/modules/VRTS/SISdk","$installdir/web/admin/htdocs/help", "$installdir/web/admin/htdocs/onlineHelp");

my %install_files =
(
    "be/etc" => {
                 "dest" => "$installdir/etc/SISdk"
             },
    "be/lib/Fabricate.pm" => {
                 "dest" => "$installdir/lib/modules/VRTS/SISdk/Fabricate.pm"
             },
    "be/lib/Util.pm" => {
                 "dest" => "$installdir/lib/modules/VRTS/SISdk/Util.pm"
             },
    "be/web/admin/cgi-bin/sisdk.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk.pl",
                 "perm" => "0744"
            },
    "be/web/admin/cgi-bin/sisdk_ui_fileUpload.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_fileUpload.pl",
                 "perm" => "0744"
            } ,       
    "be/web/admin/cgi-bin/sisdk_ui_getSummaryData.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_getSummaryData.pl",
                 "perm" => "0744"
            },     
    "be/web/admin/cgi-bin/sisdk_ui_getTestConfParameters.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_getTestConfParameters.pl",
                 "perm" => "0744"
            },        
    "be/web/admin/cgi-bin/sisdk_ui_getConfigParameter.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_getConfigParameter.pl",
                 "perm" => "0744"
            },        
    "be/web/admin/cgi-bin/sisdk_ui_getUploadedCmdScript.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_getUploadedCmdScript.pl",
                 "perm" => "0744"
            }, 
    "be/web/admin/cgi-bin/sisdk_ui_handleCapability.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_handleCapability.pl",
                 "perm" => "0744"
            },  
    "be/web/admin/cgi-bin/sisdk_ui_getDeviceIdentifiers.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_getDeviceIdentifiers.pl",
                 "perm" => "0744"
            },                                
    "be/web/admin/cgi-bin/sisdk_ui_manageAddOnConfiguration.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_manageAddOnConfiguration.pl",
                 "perm" => "0744"
            },     
    "be/web/admin/cgi-bin/sisdk_ui_downloadFile.pl" => {
                 "dest" => "$installdir/web/admin/cgi-bin/sisdk_ui_downloadFile.pl",
                 "perm" => "0744"
            },     
    "be/web/admin/htdocs/sisdk.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk.html",
                 "perm" => "0744"
            },        
    "be/web/admin/htdocs/sisdk_capability.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_capability.html",
                 "perm" => "0744"
            },        
    "be/web/admin/htdocs/sisdk_deviceIdentifer.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_deviceIdentifer.html",
                 "perm" => "0744"
            },        
    "be/web/admin/htdocs/sisdk_cliParameter.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_cliParameter.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_confirmation.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_confirmation.html",
                 "perm" => "0744"
            },            
    "be/web/admin/htdocs/sisdk_downloadAddon.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_downloadAddon.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_test.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_test.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_testResult.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_testResult.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_uploadScript.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_uploadScript.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_addon.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_addon.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_uploadAddon.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_uploadAddon.html",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/sisdk_uploadAddonToTest.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_uploadAddonToTest.html",
                 "perm" => "0744"
            },        
     "be/web/admin/htdocs/sisdk_error.html" => {
                 "dest" => "$installdir/web/admin/htdocs/sisdk_error.html",
                 "perm" => "0744"
            },       
    "be/web/admin/htdocs/help/discoveryScriptHelp.html" => {
                 "dest" => "$installdir/web/admin/htdocs/help/discoveryScriptHelp.html",
                 "perm" => "0744"
            },       
    "be/web/admin/htdocs/help/physicalDiskHelp.html" => {
                 "dest" => "$installdir/web/admin/htdocs/help/physicalDiskHelp.html",
                 "perm" => "0744"
            },                                      
    "be/web/admin/htdocs/images/Leftpanel.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/Leftpanel.gif",
                 "perm" => "0744"
            },
    "be/web/admin/htdocs/images/Leftpanel_top.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/Leftpanel_top.gif",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/Logo.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/Logo.gif",
                 "perm" => "0744"
            },           
   "be/web/admin/htdocs/images/Product-name.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/Product-name.png",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/active_light.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/active_light.gif",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/add_tab.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/add_tab.gif",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/buttonbg.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/buttonbg.png",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/expandButton.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/expandButton.png",
                 "perm" => "0744"
            },
   "be/web/admin/htdocs/images/footer.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/footer.png",
                 "perm" => "0744"
            },            
   "be/web/admin/htdocs/images/header_repeat.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/header_repeat.gif",
                 "perm" => "0744"
            },      
   "be/web/admin/htdocs/images/header.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/header.gif",
                 "perm" => "0744"
            },   
   "be/web/admin/htdocs/images/header.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/header.gif",
                 "perm" => "0744"
            },  
   "be/web/admin/htdocs/images/remove_file.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/remove_file.gif",
                 "perm" => "0744"
            },               
   "be/web/admin/htdocs/images/subtract_tab.gif" => {
                 "dest" => "$installdir/web/admin/htdocs/images/subtract_tab.gif",
                 "perm" => "0744"
            },      
   "be/web/admin/htdocs/images/help-ico.gif" => {
	                 "dest" => "$installdir/web/admin/htdocs/images/help-ico.gif",
	                 "perm" => "0744"
	            },    
   "be/web/admin/htdocs/images/add_tab_inactive.gif" => {
	                 "dest" => "$installdir/web/admin/htdocs/images/add_tab_inactive.gif",
	                 "perm" => "0744"
	            },    
  "be/web/admin/htdocs/images/subtract_tab_inactive.gif" => {
	                 "dest" => "$installdir/web/admin/htdocs/images/subtract_tab_inactive.gif",
	                 "perm" => "0744"
	            },    	            
   "be/web/admin/htdocs/images/symantec-logo.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/symantec-logo.png",
                 "perm" => "0744"
            },    
   "be/web/admin/htdocs/images/checkedBox.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/checkedBox.png",
                 "perm" => "0744"
            },  
   "be/web/admin/htdocs/images/physicalDisk.png" => {
                 "dest" => "$installdir/web/admin/htdocs/images/physicalDisk.png",
                 "perm" => "0744"
            },  
  "be/web/admin/htdocs/js/sisdk.js" => {
                 "dest" => "$installdir/web/admin/htdocs/js/sisdk.js",
                 "perm" => "0744"
            },
  "be/web/admin/htdocs/css/stylesheet_sdk.css" => {
                 "dest" => "$installdir/web/admin/htdocs/css/stylesheet_sdk.css",
                 "perm" => "0744"
            },
  "be/web/admin/htdocs/onlineHelp" => {
                 "dest" => "$installdir/web/admin/htdocs/onlineHelp"
             },            
);

VRTS::Logger::log ('debug', "SDK setup operation is $opts{operation}");
$ret = get_env();
goto out if ($ret);

if ($opts{operation} eq "install")
{
    if ($si_installed & $si_ver =~ /^4\./)
    {
        $msg = "SI Add-on version $si_ver found installed on this host.\n".
               "SI SDK cannot be installed as it is not compatible with this version of SI Add-on\n";
        print STDERR $msg;
        VRTS::Logger::log ('error', $msg);
        exit(1);
    }
    $ret = setup_bits();
}
elsif ($opts{operation} eq "uninstall")
{
    # Remove SIPlugin components 
    # they will be present if any si plugin was tested on this box
    $install_files{"lib-plugin"} =  {
         "dest" => "$installdir/lib/modules/VRTS/Enclosure/SIPlugin",
    };
    $install_files{"etc-plugin"} =  {
         "dest" => "$installdir/etc/SIPlugin",
    };
    if ($si_installed == 0)
    {
        # Remove SIBase components if SI add-on is not installed
        $install_files{"lib-sibase"} =  {
           "dest" => "$installdir/lib/modules/VRTS/Enclosure",
        };
        $install_files{probe_oper} =  {
           "dest" => "$installdir/operations/probe_array",
        };
        $install_files{probe_job} =  {
           "dest" => "$installdir/operations/jobs/probe_array.pm",
        };
        $install_files{"driver"} = {
           "dest" => "$installdir/bin/array_driver.pl"
        };
        # TODO remove SI constants. No harm if they are not removed
    }

    $ret = unsetup_bits();
}
else
{
    print STDERR "Invalid operation specified\n";
    exit(1);
}

out:
if ($ret)
{
    $msg = "Failed to $opts{operation} the SI SDK Add-on. Refer sisdk.log for details\n";
    print STDERR $msg;
    VRTS::Logger::log ('error', $msg);
    exit(1);
}
else
{
    $msg = "$opts{operation} of SI SDK add-on executed successfully\n";
    print STDOUT $msg;
    VRTS::Logger::log ('debug', $msg);
    exit(0);
}

sub setup_bits
{
    my $ret = 0;

    foreach my $dir (@install_dirs)
    {
        eval ( File::Path::mkpath($dir, 0, 0755) );
        if ($@)
        {
            $msg = "setup_bits: Failed to create directory $dir: $@\n";
            VRTS::Logger::log ('error', $msg);
            return 1;
        }
    }

    foreach my $file (keys %install_files)
    {
        if (-f $file)
        {
            VRTS::Logger::log ('debug', "Installing $file at $install_files{$file}->{dest}");
            $ret = File::Copy::copy ($file, $install_files{$file}->{dest});
            if ($ret == 0)
            {
                $msg = "Failed to copy file $file: $!\n";
                VRTS::Logger::log ('error', $msg);
                return 1;
            }
            chmod (oct($install_files{$file}->{perm}),$install_files{$file}->{dest})
                  if (defined $install_files{$file}->{perm});
           
         }
         elsif (-d $file)
         {
             my $dest = $install_files{$file}->{dest};
             File::Path::rmtree($dest);

             VRTS::Logger::log ('debug', "Installing $file at $dest");
             eval { VRTS::Util::copy_recursively($file, $dest)};
             if ($@)
             {
                 $msg = "Failed to copy directory $file";
                 VRTS::Logger::log ('error', $msg);
                 return 1;
             }
        }
    }
    return 0;
}

sub unsetup_bits
{
    foreach my $file (keys %install_files)
    {
        my $dest = $install_files{$file}->{dest};
        if (-f $dest)
        {
            VRTS::Logger::log ('debug', "Removing file $dest");
            unlink ($dest);
        }
        elsif (-d $dest)
        {
             VRTS::Logger::log ('debug', "Removing dir $dest");
             File::Path::rmtree($dest);
        }
    }
    return 0;
}

sub get_env
{
    my $ret = 0;
    my $info = new VRTS::AddOnInfo;
    my $ret = $info->initialize();
    if($ret == 0)
    {
        my %f_info = %{$info->get_features()};
        for my $feature (keys %f_info)
        {
             if ($f_info{$feature}{'ID'} =~ /^VRTSsfmsi-/)
             {
                 $si_installed = 1;
                 $si_ver = $f_info{$feature}{'VERSION'};
                 VRTS::Logger::log('debug', "SI Add-on version $si_ver installed");
                 last;
             }
         }
    }
    return 0;
}

